"""
High level functions that deal with the generation of the disassembled text
lines.

This file also contains definitions for the syntax highlighting.

Finally there are functions that deal with anterior/posterior user-defined
lines."""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _ida_lines
else:
    import _ida_lines

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

SWIG_PYTHON_LEGACY_BOOL = _ida_lines.SWIG_PYTHON_LEGACY_BOOL

import ida_idaapi

COLOR_ON = _ida_lines.COLOR_ON
r"""
Escape character (ON). Followed by a color code (color_t).
"""

COLOR_OFF = _ida_lines.COLOR_OFF
r"""
Escape character (OFF). Followed by a color code (color_t).
"""

COLOR_ESC = _ida_lines.COLOR_ESC
r"""
Escape character (Quote next character). This is needed to output '\1' and '\2'
characters.
"""

COLOR_INV = _ida_lines.COLOR_INV
r"""
Escape character (Inverse foreground and background colors). This escape
character has no corresponding COLOR_OFF. Its action continues until the next
COLOR_INV or end of line.
"""

SCOLOR_ON = _ida_lines.SCOLOR_ON
r"""
Escape character (ON)
"""

SCOLOR_OFF = _ida_lines.SCOLOR_OFF
r"""
Escape character (OFF)
"""

SCOLOR_ESC = _ida_lines.SCOLOR_ESC
r"""
Escape character (Quote next character)
"""

SCOLOR_INV = _ida_lines.SCOLOR_INV
r"""
Escape character (Inverse colors)
"""

SCOLOR_DEFAULT = _ida_lines.SCOLOR_DEFAULT
r"""
Default.
"""

SCOLOR_REGCMT = _ida_lines.SCOLOR_REGCMT
r"""
Regular comment.
"""

SCOLOR_RPTCMT = _ida_lines.SCOLOR_RPTCMT
r"""
Repeatable comment (defined not here)
"""

SCOLOR_AUTOCMT = _ida_lines.SCOLOR_AUTOCMT
r"""
Automatic comment.
"""

SCOLOR_INSN = _ida_lines.SCOLOR_INSN
r"""
Instruction.
"""

SCOLOR_DATNAME = _ida_lines.SCOLOR_DATNAME
r"""
Dummy Data Name.
"""

SCOLOR_DNAME = _ida_lines.SCOLOR_DNAME
r"""
Regular Data Name.
"""

SCOLOR_DEMNAME = _ida_lines.SCOLOR_DEMNAME
r"""
Demangled Name.
"""

SCOLOR_SYMBOL = _ida_lines.SCOLOR_SYMBOL
r"""
Punctuation.
"""

SCOLOR_CHAR = _ida_lines.SCOLOR_CHAR
r"""
Char constant in instruction.
"""

SCOLOR_STRING = _ida_lines.SCOLOR_STRING
r"""
String constant in instruction.
"""

SCOLOR_NUMBER = _ida_lines.SCOLOR_NUMBER
r"""
Numeric constant in instruction.
"""

SCOLOR_VOIDOP = _ida_lines.SCOLOR_VOIDOP
r"""
Void operand.
"""

SCOLOR_CREF = _ida_lines.SCOLOR_CREF
r"""
Code reference.
"""

SCOLOR_DREF = _ida_lines.SCOLOR_DREF
r"""
Data reference.
"""

SCOLOR_CREFTAIL = _ida_lines.SCOLOR_CREFTAIL
r"""
Code reference to tail byte.
"""

SCOLOR_DREFTAIL = _ida_lines.SCOLOR_DREFTAIL
r"""
Data reference to tail byte.
"""

SCOLOR_ERROR = _ida_lines.SCOLOR_ERROR
r"""
Error or problem.
"""

SCOLOR_PREFIX = _ida_lines.SCOLOR_PREFIX
r"""
Line prefix.
"""

SCOLOR_BINPREF = _ida_lines.SCOLOR_BINPREF
r"""
Binary line prefix bytes.
"""

SCOLOR_EXTRA = _ida_lines.SCOLOR_EXTRA
r"""
Extra line.
"""

SCOLOR_ALTOP = _ida_lines.SCOLOR_ALTOP
r"""
Alternative operand.
"""

SCOLOR_HIDNAME = _ida_lines.SCOLOR_HIDNAME
r"""
Hidden name.
"""

SCOLOR_LIBNAME = _ida_lines.SCOLOR_LIBNAME
r"""
Library function name.
"""

SCOLOR_LOCNAME = _ida_lines.SCOLOR_LOCNAME
r"""
Local variable name.
"""

SCOLOR_CODNAME = _ida_lines.SCOLOR_CODNAME
r"""
Dummy code name.
"""

SCOLOR_ASMDIR = _ida_lines.SCOLOR_ASMDIR
r"""
Assembler directive.
"""

SCOLOR_MACRO = _ida_lines.SCOLOR_MACRO
r"""
Macro.
"""

SCOLOR_DSTR = _ida_lines.SCOLOR_DSTR
r"""
String constant in data directive.
"""

SCOLOR_DCHAR = _ida_lines.SCOLOR_DCHAR
r"""
Char constant in data directive.
"""

SCOLOR_DNUM = _ida_lines.SCOLOR_DNUM
r"""
Numeric constant in data directive.
"""

SCOLOR_KEYWORD = _ida_lines.SCOLOR_KEYWORD
r"""
Keywords.
"""

SCOLOR_REG = _ida_lines.SCOLOR_REG
r"""
Register name.
"""

SCOLOR_IMPNAME = _ida_lines.SCOLOR_IMPNAME
r"""
Imported name.
"""

SCOLOR_SEGNAME = _ida_lines.SCOLOR_SEGNAME
r"""
Segment name.
"""

SCOLOR_UNKNAME = _ida_lines.SCOLOR_UNKNAME
r"""
Dummy unknown name.
"""

SCOLOR_CNAME = _ida_lines.SCOLOR_CNAME
r"""
Regular code name.
"""

SCOLOR_UNAME = _ida_lines.SCOLOR_UNAME
r"""
Regular unknown name.
"""

SCOLOR_COLLAPSED = _ida_lines.SCOLOR_COLLAPSED
r"""
Collapsed line.
"""

SCOLOR_ADDR = _ida_lines.SCOLOR_ADDR
r"""
Hidden address mark.
"""

COLOR_SELECTED = _ida_lines.COLOR_SELECTED
r"""
Selected.
"""

COLOR_LIBFUNC = _ida_lines.COLOR_LIBFUNC
r"""
Library function.
"""

COLOR_REGFUNC = _ida_lines.COLOR_REGFUNC
r"""
Regular function.
"""

COLOR_CODE = _ida_lines.COLOR_CODE
r"""
Single instruction.
"""

COLOR_DATA = _ida_lines.COLOR_DATA
r"""
Data bytes.
"""

COLOR_UNKNOWN = _ida_lines.COLOR_UNKNOWN
r"""
Unexplored byte.
"""

COLOR_EXTERN = _ida_lines.COLOR_EXTERN
r"""
External name definition segment.
"""

COLOR_CURITEM = _ida_lines.COLOR_CURITEM
r"""
Current item.
"""

COLOR_CURLINE = _ida_lines.COLOR_CURLINE
r"""
Current line.
"""

COLOR_HIDLINE = _ida_lines.COLOR_HIDLINE
r"""
Hidden line.
"""

COLOR_LUMFUNC = _ida_lines.COLOR_LUMFUNC
r"""
Lumina function.
"""

COLOR_BG_MAX = _ida_lines.COLOR_BG_MAX
r"""
Max color number.
"""


def tag_strlen(*args) -> "ssize_t":
    r"""
    tag_strlen(line) -> ssize_t
    Calculate length of a colored string This function computes the length in
    unicode codepoints of a line

    @param line: (C++: const char *) char const *
    @return: the number of codepoints in the line, or -1 on error
    """
    return _ida_lines.tag_strlen(*args)

def calc_prefix_color(*args) -> "color_t":
    r"""
    calc_prefix_color(ea) -> color_t
    Get prefix color for line at 'ea'

    @param ea: (C++: ea_t)
    @return: Line prefix colors
    """
    return _ida_lines.calc_prefix_color(*args)

def calc_bg_color(*args) -> "bgcolor_t":
    r"""
    calc_bg_color(ea) -> bgcolor_t
    Get background color for line at 'ea'

    @param ea: (C++: ea_t)
    @return: RGB color
    """
    return _ida_lines.calc_bg_color(*args)

def add_sourcefile(*args) -> "bool":
    r"""
    add_sourcefile(ea1, ea2, filename) -> bool
    Mark a range of address as belonging to a source file. An address range may
    belong only to one source file. A source file may be represented by several
    address ranges.

    @param ea1: (C++: ea_t) linear address of start of the address range
    @param ea2: (C++: ea_t) linear address of end of the address range (excluded)
    @param filename: (C++: const char *) name of source file.
    @return: success
    """
    return _ida_lines.add_sourcefile(*args)

def get_sourcefile(*args) -> "char const *":
    r"""
    get_sourcefile(ea, bounds=None) -> char const *
    Get name of source file occupying the given address.

    @param ea: (C++: ea_t) linear address
    @param bounds: (C++: range_t *) pointer to the output buffer with the address range for the
                   current file. May be nullptr.
    @return: nullptr if source file information is not found, otherwise returns
             pointer to file name
    """
    return _ida_lines.get_sourcefile(*args)

def del_sourcefile(*args) -> "bool":
    r"""
    del_sourcefile(ea) -> bool
    Delete information about the source file.

    @param ea: (C++: ea_t) linear address
    @return: success
    """
    return _ida_lines.del_sourcefile(*args)

def install_user_defined_prefix(*args) -> "bool":
    r"""
    install_user_defined_prefix(prefix_len, udp, owner) -> bool
    User-defined line-prefixes are displayed just after the autogenerated line
    prefixes in the disassembly listing. There is no need to call this function
    explicitly. Use the user_defined_prefix_t class.

    @param prefix_len: (C++: size_t) prefixed length. if 0, then uninstall UDP
    @param udp: (C++: struct user_defined_prefix_t *) object to generate user-defined prefix
    @param owner: (C++: const void *) pointer to the plugin_t that owns UDP if non-nullptr, then the
                  object will be uninstalled and destroyed when the plugin gets
                  unloaded
    """
    return _ida_lines.install_user_defined_prefix(*args)
class user_defined_prefix_t(object):
    r"""
    Proxy of C++ user_defined_prefix_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(self, prefix_len, owner) -> user_defined_prefix_t

        @param prefix_len: size_t
        @param owner: void const *
        """
        if self.__class__ == user_defined_prefix_t:
            _self = None
        else:
            _self = self
        _ida_lines.user_defined_prefix_t_swiginit(self, _ida_lines.new_user_defined_prefix_t(_self, *args))
    __swig_destroy__ = _ida_lines.delete_user_defined_prefix_t

    def get_user_defined_prefix(self, *args) -> "void":
        r"""
        get_user_defined_prefix(self, ea, insn, lnnum, indent, line)
        This callback must be overridden by the derived class.

        @param ea: (C++: ea_t) the current address
        @param insn: (C++: const class insn_t &) the current instruction. if the current item is not an instruction,
                     then insn.itype is zero.
        @param lnnum: (C++: int) number of the current line (each address may have several listing
                      lines for it). 0 means the very first line for the current
                      address.
        @param indent: (C++: int) see explanations for gen_printf()
        @param line: (C++: const char *) the line to be generated. the line usually contains color tags.
                     this argument can be examined to decide whether to generate the
                     prefix.
        """
        return _ida_lines.user_defined_prefix_t_get_user_defined_prefix(self, *args)
    def __disown__(self):
        self.this.disown()
        _ida_lines.disown_user_defined_prefix_t(self)
        return weakref.proxy(self)

# Register user_defined_prefix_t in _ida_lines:
_ida_lines.user_defined_prefix_t_swigregister(user_defined_prefix_t)
cvar = _ida_lines.cvar
COLOR_DEFAULT = cvar.COLOR_DEFAULT
r"""
Default.
"""
COLOR_REGCMT = cvar.COLOR_REGCMT
r"""
Regular comment.
"""
COLOR_RPTCMT = cvar.COLOR_RPTCMT
r"""
Repeatable comment (comment defined somewhere else)
"""
COLOR_AUTOCMT = cvar.COLOR_AUTOCMT
r"""
Automatic comment.
"""
COLOR_INSN = cvar.COLOR_INSN
r"""
Instruction.
"""
COLOR_DATNAME = cvar.COLOR_DATNAME
r"""
Dummy Data Name.
"""
COLOR_DNAME = cvar.COLOR_DNAME
r"""
Regular Data Name.
"""
COLOR_DEMNAME = cvar.COLOR_DEMNAME
r"""
Demangled Name.
"""
COLOR_SYMBOL = cvar.COLOR_SYMBOL
r"""
Punctuation.
"""
COLOR_CHAR = cvar.COLOR_CHAR
r"""
Char constant in instruction.
"""
COLOR_STRING = cvar.COLOR_STRING
r"""
String constant in instruction.
"""
COLOR_NUMBER = cvar.COLOR_NUMBER
r"""
Numeric constant in instruction.
"""
COLOR_VOIDOP = cvar.COLOR_VOIDOP
r"""
Void operand.
"""
COLOR_CREF = cvar.COLOR_CREF
r"""
Code reference.
"""
COLOR_DREF = cvar.COLOR_DREF
r"""
Data reference.
"""
COLOR_CREFTAIL = cvar.COLOR_CREFTAIL
r"""
Code reference to tail byte.
"""
COLOR_DREFTAIL = cvar.COLOR_DREFTAIL
r"""
Data reference to tail byte.
"""
COLOR_ERROR = cvar.COLOR_ERROR
r"""
Error or problem.
"""
COLOR_PREFIX = cvar.COLOR_PREFIX
r"""
Line prefix.
"""
COLOR_BINPREF = cvar.COLOR_BINPREF
r"""
Binary line prefix bytes.
"""
COLOR_EXTRA = cvar.COLOR_EXTRA
r"""
Extra line.
"""
COLOR_ALTOP = cvar.COLOR_ALTOP
r"""
Alternative operand.
"""
COLOR_HIDNAME = cvar.COLOR_HIDNAME
r"""
Hidden name.
"""
COLOR_LIBNAME = cvar.COLOR_LIBNAME
r"""
Library function name.
"""
COLOR_LOCNAME = cvar.COLOR_LOCNAME
r"""
Local variable name.
"""
COLOR_CODNAME = cvar.COLOR_CODNAME
r"""
Dummy code name.
"""
COLOR_ASMDIR = cvar.COLOR_ASMDIR
r"""
Assembler directive.
"""
COLOR_MACRO = cvar.COLOR_MACRO
r"""
Macro.
"""
COLOR_DSTR = cvar.COLOR_DSTR
r"""
String constant in data directive.
"""
COLOR_DCHAR = cvar.COLOR_DCHAR
r"""
Char constant in data directive.
"""
COLOR_DNUM = cvar.COLOR_DNUM
r"""
Numeric constant in data directive.
"""
COLOR_KEYWORD = cvar.COLOR_KEYWORD
r"""
Keywords.
"""
COLOR_REG = cvar.COLOR_REG
r"""
Register name.
"""
COLOR_IMPNAME = cvar.COLOR_IMPNAME
r"""
Imported name.
"""
COLOR_SEGNAME = cvar.COLOR_SEGNAME
r"""
Segment name.
"""
COLOR_UNKNAME = cvar.COLOR_UNKNAME
r"""
Dummy unknown name.
"""
COLOR_CNAME = cvar.COLOR_CNAME
r"""
Regular code name.
"""
COLOR_UNAME = cvar.COLOR_UNAME
r"""
Regular unknown name.
"""
COLOR_COLLAPSED = cvar.COLOR_COLLAPSED
r"""
Collapsed line.
"""
COLOR_FG_MAX = cvar.COLOR_FG_MAX
r"""
Max color number.
"""
COLOR_ADDR = cvar.COLOR_ADDR
r"""
hidden address marks. the address is represented as 8digit hex number: 01234567.
it doesn't have COLOR_OFF pair. NB: for 64-bit IDA, the address is 16digit.
"""
COLOR_OPND1 = cvar.COLOR_OPND1
r"""
Instruction operand 1.
"""
COLOR_OPND2 = cvar.COLOR_OPND2
r"""
Instruction operand 2.
"""
COLOR_OPND3 = cvar.COLOR_OPND3
r"""
Instruction operand 3.
"""
COLOR_OPND4 = cvar.COLOR_OPND4
r"""
Instruction operand 4.
"""
COLOR_OPND5 = cvar.COLOR_OPND5
r"""
Instruction operand 5.
"""
COLOR_OPND6 = cvar.COLOR_OPND6
r"""
Instruction operand 6.
"""
COLOR_OPND7 = cvar.COLOR_OPND7
r"""
Instruction operand 7.
"""
COLOR_OPND8 = cvar.COLOR_OPND8
r"""
Instruction operand 8.
"""
COLOR_RESERVED1 = cvar.COLOR_RESERVED1
r"""
This tag is reserved for internal IDA use.
"""
COLOR_LUMINA = cvar.COLOR_LUMINA
r"""
Lumina-related, only for the navigation band.
"""

VEL_POST = _ida_lines.VEL_POST

VEL_CMT = _ida_lines.VEL_CMT


def add_extra_line(*args) -> "bool":
    r"""
    add_extra_line(ea, isprev, format) -> bool
    Add anterior/posterior non-comment line(s).

    @param ea: (C++: ea_t) linear address
    @param isprev: (C++: bool) do we add anterior lines? (0-no, posterior)
    @param format: (C++: const char *) printf() style format string. may contain \n to denote new lines.
    @return: true if success
    """
    return _ida_lines.add_extra_line(*args)

def add_extra_cmt(*args) -> "bool":
    r"""
    add_extra_cmt(ea, isprev, format) -> bool
    Add anterior/posterior comment line(s).

    @param ea: (C++: ea_t) linear address
    @param isprev: (C++: bool) do we add anterior lines? (0-no, posterior)
    @param format: (C++: const char *) printf() style format string. may contain \n to denote new lines.
                   The resulting string should not contain comment characters (;),
                   the kernel will add them automatically.
    @return: true if success
    """
    return _ida_lines.add_extra_cmt(*args)

def add_pgm_cmt(*args) -> "bool":
    r"""
    add_pgm_cmt(format) -> bool
    Add anterior comment line(s) at the start of program.

    @param format: (C++: const char *) printf() style format string. may contain \n to denote new lines.
                   The resulting string should not contain comment characters (;),
                   the kernel will add them automatically.
    @return: true if success
    """
    return _ida_lines.add_pgm_cmt(*args)
GDISMF_AS_STACK = _ida_lines.GDISMF_AS_STACK

GDISMF_ADDR_TAG = _ida_lines.GDISMF_ADDR_TAG


def generate_disasm_line(*args) -> "qstring *":
    r"""
    generate_disasm_line(ea, flags=0) -> str

    @param ea: ea_t
    @param flags: int
    """
    return _ida_lines.generate_disasm_line(*args)
GENDSM_FORCE_CODE = _ida_lines.GENDSM_FORCE_CODE

GENDSM_MULTI_LINE = _ida_lines.GENDSM_MULTI_LINE

GENDSM_REMOVE_TAGS = _ida_lines.GENDSM_REMOVE_TAGS


def get_first_free_extra_cmtidx(*args) -> "int":
    r"""
    get_first_free_extra_cmtidx(ea, start) -> int

    @param ea: ea_t
    @param start: int
    """
    return _ida_lines.get_first_free_extra_cmtidx(*args)

def update_extra_cmt(*args) -> "void":
    r"""
    update_extra_cmt(ea, what, str)

    @param ea: ea_t
    @param what: int
    @param str: char const *
    """
    return _ida_lines.update_extra_cmt(*args)

def del_extra_cmt(*args) -> "void":
    r"""
    del_extra_cmt(ea, what)

    @param ea: ea_t
    @param what: int
    """
    return _ida_lines.del_extra_cmt(*args)

def get_extra_cmt(*args) -> "int":
    r"""
    get_extra_cmt(ea, what) -> ssize_t

    @param ea: ea_t
    @param what: int
    """
    return _ida_lines.get_extra_cmt(*args)

def delete_extra_cmts(*args) -> "void":
    r"""
    delete_extra_cmts(ea, what)

    @param ea: ea_t
    @param what: int
    """
    return _ida_lines.delete_extra_cmts(*args)

def create_encoding_helper(*args) -> "encoder_t *":
    r"""
    create_encoding_helper(encidx=-1, nr=nr_once) -> encoder_t *

    @param encidx: int
    @param nr: enum encoder_t::notify_recerr_t
    """
    return _ida_lines.create_encoding_helper(*args)

def tag_remove(*args) -> "PyObject *":
    r"""
    tag_remove(nonnul_instr) -> str

    @param nonnul_instr: char const *
    """
    return _ida_lines.tag_remove(*args)

def set_user_defined_prefix(*args) -> "PyObject *":
    r"""
    set_user_defined_prefix(width, pycb) -> bool
    Deprecated. Please use install_user_defined_prefix() instead

    @param width: size_t
    @param pycb: PyObject *
    """
    return _ida_lines.set_user_defined_prefix(*args)

def tag_addr(*args) -> "PyObject *":
    r"""
    tag_addr(ea) -> PyObject *
    Insert an address mark into a string.

    @param ea: (C++: ea_t) address to include
    """
    return _ida_lines.tag_addr(*args)

def tag_skipcode(*args) -> "int":
    r"""
    tag_skipcode(line) -> int
    Skip one color code. This function should be used if you are interested in color
    codes and want to analyze all of them. Otherwise tag_skipcodes() function is
    better since it will skip all colors at once. This function will skip the
    current color code if there is one. If the current symbol is not a color code,
    it will return the input.

    @param line: (C++: const char *) char const *
    @return: moved pointer
    """
    return _ida_lines.tag_skipcode(*args)

def tag_skipcodes(*args) -> "int":
    r"""
    tag_skipcodes(line) -> int
    Move the pointer past all color codes.

    @param line: (C++: const char *) can't be nullptr
    @return: moved pointer, can't be nullptr
    """
    return _ida_lines.tag_skipcodes(*args)

def tag_advance(*args) -> "int":
    r"""
    tag_advance(line, cnt) -> int
    Move pointer to a 'line' to 'cnt' positions right. Take into account escape
    sequences.

    @param line: (C++: const char *) pointer to string
    @param cnt: (C++: int) number of positions to move right
    @return: moved pointer
    """
    return _ida_lines.tag_advance(*args)

def generate_disassembly(*args) -> "PyObject *":
    r"""
    generate_disassembly(ea, max_lines, as_stack, notags) -> (int, [str, ...])
    Generate disassembly lines (many lines) and put them into a buffer

    @param ea: address to generate disassembly for
    @param max_lines: how many lines max to generate
    @param as_stack: Display undefined items as 2/4/8 bytes
    @param notags: bool
    @return:     - None on failure
        - tuple(most_important_line_number, list(lines)) : Returns a tuple containing
          the most important line number and a list of generated lines
    """
    return _ida_lines.generate_disassembly(*args)

#<pycode(py_lines)>
import _ida_idaapi
import _ida_lines

# ---------------- Color escape sequence defitions -------------------------
COLOR_ADDR_SIZE = 16 if _ida_idaapi.BADADDR == 0xFFFFFFFFFFFFFFFF else 8
r"""
Size of a tagged address (see COLOR_ADDR)
"""
SCOLOR_FG_MAX   = '\x28'             #  Max color number
cvar = _ida_lines.cvar
SCOLOR_OPND1    = chr(cvar.COLOR_ADDR+1)  #  Instruction operand 1
SCOLOR_OPND2    = chr(cvar.COLOR_ADDR+2)  #  Instruction operand 2
SCOLOR_OPND3    = chr(cvar.COLOR_ADDR+3)  #  Instruction operand 3
SCOLOR_OPND4    = chr(cvar.COLOR_ADDR+4)  #  Instruction operand 4
SCOLOR_OPND5    = chr(cvar.COLOR_ADDR+5)  #  Instruction operand 5
SCOLOR_OPND6    = chr(cvar.COLOR_ADDR+6)  #  Instruction operand 6
SCOLOR_UTF8     = chr(cvar.COLOR_ADDR+10) #  Following text is UTF-8 encoded

# ---------------- Line prefix colors --------------------------------------
PALETTE_SIZE   =  (cvar.COLOR_FG_MAX+_ida_lines.COLOR_BG_MAX)

def requires_color_esc(c):
    r"""
    Checks if the given character requires escaping
    Is the given char a color escape character?
    """
    t = ord(c[0])
    return c >= COLOR_ON and c <= COLOR_INV

def COLSTR(str, tag):
    r"""
    Utility function to create a colored line
    @param str: The string
    @param tag: Color tag constant. One of SCOLOR_XXXX
    """
    return SCOLOR_ON + tag + str + SCOLOR_OFF + tag

#</pycode(py_lines)>


E_PREV = cvar.E_PREV
E_NEXT = cvar.E_NEXT





"""
summary: programmatically drive a debugging session

description:
  Start a debugging session, step through the first five
  instructions. Each instruction is disassembled after
  execution.
"""

import ida_dbg
import ida_ida
import ida_lines

class MyDbgHook(ida_dbg.DBG_Hooks):
    """ Own debug hook class that implementd the callback functions """

    def __init__(self):
        ida_dbg.DBG_Hooks.__init__(self) # important
        self.steps = 0

    def log(self, msg):
        print(">>> %s" % msg)

    def dbg_process_start(self, pid, tid, ea, name, base, size):
        self.log("Process started, pid=%d tid=%d name=%s" % (pid, tid, name))

    def dbg_process_exit(self, pid, tid, ea, code):
        self.log("Process exited pid=%d tid=%d ea=0x%x code=%d" % (pid, tid, ea, code))

    def dbg_library_unload(self, pid, tid, ea, info):
        self.log("Library unloaded: pid=%d tid=%d ea=0x%x info=%s" % (pid, tid, ea, info))

    def dbg_process_attach(self, pid, tid, ea, name, base, size):
        self.log("Process attach pid=%d tid=%d ea=0x%x name=%s base=%x size=%x" % (pid, tid, ea, name, base, size))

    def dbg_process_detach(self, pid, tid, ea):
        self.log("Process detached, pid=%d tid=%d ea=0x%x" % (pid, tid, ea))

    def dbg_library_load(self, pid, tid, ea, name, base, size):
        self.log("Library loaded: pid=%d tid=%d name=%s base=%x" % (pid, tid, name, base))

    def dbg_bpt(self, tid, ea):
        self.log("Break point at 0x%x pid=%d" % (ea, tid))
        # return values:
        #   -1 - to display a breakpoint warning dialog
        #        if the process is suspended.
        #    0 - to never display a breakpoint warning dialog.
        #    1 - to always display a breakpoint warning dialog.
        return 0

    def dbg_suspend_process(self):
        self.log("Process suspended")

    def dbg_exception(self, pid, tid, ea, exc_code, exc_can_cont, exc_ea, exc_info):
        self.log("Exception: pid=%d tid=%d ea=0x%x exc_code=0x%x can_continue=%d exc_ea=0x%x exc_info=%s" % (
            pid, tid, ea, exc_code & ida_idaapi.BADADDR, exc_can_cont, exc_ea, exc_info))
        # return values:
        #   -1 - to display an exception warning dialog
        #        if the process is suspended.
        #   0  - to never display an exception warning dialog.
        #   1  - to always display an exception warning dialog.
        return 0

    def dbg_trace(self, tid, ea):
        self.log("Trace tid=%d ea=0x%x" % (tid, ea))
        # return values:
        #   1  - do not log this trace event;
        #   0  - log it
        return 0

    def dbg_step_into(self):
        self.log("Step into")
        self.dbg_step_over()

    def dbg_run_to(self, pid, tid=0, ea=0):
        self.log("Runto: tid=%d, ea=%x" % (tid, ea))
        ida_dbg.request_step_over()

    def dbg_step_over(self):
        eip = ida_dbg.get_reg_val("EIP")
        disasm = ida_lines.tag_remove(
            ida_lines.generate_disasm_line(
                eip))
        self.log("Step over: EIP=0x%x, disassembly=%s" % (eip, disasm))

        self.steps += 1
        if self.steps >= 5:
            ida_dbg.request_exit_process()
        else:
            ida_dbg.request_step_over()


# Remove an existing debug hook
try:
    if debughook:
        print("Removing previous hook ...")
        debughook.unhook()
except:
    pass

# Install the debug hook
debughook = MyDbgHook()
debughook.hook()

ep = ida_ida.inf_get_start_ip()
if ida_dbg.request_run_to(ep): # Request stop at entry point
    ida_dbg.run_requests()     # Launch process
else:
    print("Impossible to prepare debugger requests. Is a debugger selected?")



"""
summary: dynamically colorize lines backgrounds (or parts of them)

description:
  shows how one can dynamically alter the lines background
  rendering (as opposed to, say, using ida_nalt.set_item_color()),
  and also shows how that rendering can be limited to just a few
  glyphs, not the whole line.
"""

import ida_kernwin
import ida_bytes

class lines_rendering_hooks_t(ida_kernwin.UI_Hooks):
    def __init__(self):
        ida_kernwin.UI_Hooks.__init__(self)

        # We'll color all lines starting with the current
        # one, with all available highlights...
        self.instantiated_at = ida_kernwin.get_screen_ea()
        self.color_info = []

        data = [
            ida_kernwin.CK_EXTRA1,
            ida_kernwin.CK_EXTRA2,
            ida_kernwin.CK_EXTRA3,
            ida_kernwin.CK_EXTRA4,
            ida_kernwin.CK_EXTRA5,
            ida_kernwin.CK_EXTRA6,
            ida_kernwin.CK_EXTRA7,
            ida_kernwin.CK_EXTRA8,
            ida_kernwin.CK_EXTRA9,
            ida_kernwin.CK_EXTRA10,
            ida_kernwin.CK_EXTRA11,
            ida_kernwin.CK_EXTRA12,
            ida_kernwin.CK_EXTRA13,
            ida_kernwin.CK_EXTRA14,
            ida_kernwin.CK_EXTRA15,
            ida_kernwin.CK_EXTRA16,
            # let's also try these colors keys, because why not
            ida_kernwin.CK_TRACE,
            ida_kernwin.CK_TRACE_OVL,
            [
                ida_kernwin.CK_TRACE,
                ida_kernwin.CK_TRACE_OVL,
            ],
        ]
        ea = self.instantiated_at
        for one in data:
            self.color_info.append((ea, one))
            ea = ida_bytes.next_head(ea, ida_idaapi.BADADDR)

        # ...and then we'll a few more things, such as
        # overriding parts of a previously-specified overlay,
        # and restricting the override to a few glyphs
        self.color_info.append(
            (
                self.color_info[6][0],
                [
                    (ida_kernwin.CK_EXTRA2, 7, 3),
                    (ida_kernwin.CK_EXTRA4, 2, 1),
                    (ida_kernwin.CK_EXTRA10, 2, 0),
                    (ida_kernwin.CK_EXTRA10, 20, 10),
                ]
            ))
        self.color_info.append(
            (
                self.color_info[7][0],
                [
                    (ida_kernwin.CK_EXTRA1, 1, 1),
                    (ida_kernwin.CK_EXTRA2, 3, 1),
                    (ida_kernwin.CK_EXTRA3, 5, 1),
                    (ida_kernwin.CK_EXTRA4, 7, 1),
                    (ida_kernwin.CK_EXTRA5, 9, 1),
                    (ida_kernwin.CK_EXTRA6, 11, 1),
                    (ida_kernwin.CK_EXTRA7, 13, 1),
                    (ida_kernwin.CK_EXTRA8, 15, 1),
                    (ida_kernwin.CK_EXTRA9, 17, 1),
                    (ida_kernwin.CK_EXTRA10, 19, 1),
                    (ida_kernwin.CK_EXTRA11, 21, 1),
                    (ida_kernwin.CK_EXTRA12, 23, 1),
                    (ida_kernwin.CK_EXTRA13, 25, 1),
                    (ida_kernwin.CK_EXTRA14, 27, 1),
                    (ida_kernwin.CK_EXTRA15, 29, 1),
                    (ida_kernwin.CK_EXTRA16, 31, 1),
                ]
            ))
        self.color_info.append(
            (
                self.color_info[8][0],
                [
                    (ida_kernwin.CK_EXTRA1, 16, 45),
                    (ida_kernwin.CK_EXTRA2, 19, 45),
                    (ida_kernwin.CK_EXTRA3, 22, 45),
                    (ida_kernwin.CK_EXTRA4, 25, 45),
                ]
            ))


    def get_lines_rendering_info(self, out, widget, rin):
        for section_lines in rin.sections_lines:
            for line in section_lines:
                line_ea = line.at.toea()
                for ea, directives in self.color_info:
                    if ea == line_ea:
                        if not isinstance(directives, list):
                            directives = [directives]
                        for directive in directives:
                            e = ida_kernwin.line_rendering_output_entry_t(line)
                            if isinstance(directive, tuple):
                                color, cpx, nchars = directive
                                e.bg_color = color
                                e.cpx = cpx
                                e.nchars = nchars
                                e.flags |= ida_kernwin.LROEF_CPS_RANGE
                            else:
                                e.bg_color = directive
                            out.entries.push_back(e)


lrh = lines_rendering_hooks_t()
lrh.hook()

# Force a refresh of IDA View-A
ida_kernwin.refresh_idaview_anyway()



"""
summary: using the low-level tracing hook

description:
  This script demonstrates using the low-level tracing hook
  (ida_dbg.DBG_Hooks.dbg_trace). It can be run like so:

       ida[t].exe -B -Sdbg_trace.py -Ltrace.log file.exe
"""

import time

import ida_dbg
import ida_ida
import ida_pro
import ida_ua
from ida_allins import NN_callni, NN_call, NN_callfi
from ida_lines import generate_disasm_line, GENDSM_FORCE_CODE, GENDSM_REMOVE_TAGS

# Note: this try/except block below is just there to
# let us (at Hex-Rays) test this script in various
# situations.
try:
    import idc
    print(idc.ARGV[1])
    under_test = bool(idc.ARGV[1])
except:
    under_test = False

class TraceHook(ida_dbg.DBG_Hooks):
    def __init__(self):
        ida_dbg.DBG_Hooks.__init__(self)
        self.traces = 0
        self.epReached = False

    def _log(self, msg):
        print(">>> %s" % msg)

    def dbg_trace(self, tid, ea):
        # Log all traced addresses
        if ea < ida_ida.inf_get_min_ea() or ea > ida_ida.inf_get_max_ea():
            raise Exception(
                "Received a trace callback for an address outside this database!"
            )

        self._log("trace %08X" % ea)
        self.traces += 1
        insn = ida_ua.insn_t()
        insnlen = ida_ua.decode_insn(insn, ea)
        # log disassembly and ESP for call instructions
        if insnlen > 0 and insn.itype in [NN_callni, NN_call, NN_callfi]:
            self._log(
                "call insn: %s"
                % generate_disasm_line(ea, GENDSM_FORCE_CODE | GENDSM_REMOVE_TAGS)
            )
            self._log("ESP=%08X" % ida_dbg.get_reg_val("ESP"))

        return 1

    def dbg_run_to(self, pid, tid=0, ea=0):
        # this hook is called once execution reaches temporary breakpoint set by run_to(ep) below
        if not self.epReached:
            ida_dbg.refresh_debugger_memory()
            self._log("reached entry point at 0x%X" % ida_dbg.get_reg_val("EIP"))
            self._log("current step trace options: %x" % ida_dbg.get_step_trace_options())
            self.epReached = True

        # enable step tracing (single-step the program and generate dbg_trace events)
        ida_dbg.request_enable_step_trace(1)
        # change options to only "over debugger segments" (i.e. library functions will be traced)
        ida_dbg.request_set_step_trace_options(ida_dbg.ST_OVER_DEBUG_SEG)
        ida_dbg.request_continue_process()
        ida_dbg.run_requests()

    def dbg_process_exit(self, pid, tid, ea, code):
        self._log("process exited with %d" % code)
        self._log("traced %d instructions" % self.traces)
        return 0


def do_trace(then_quit_ida=True):
    debugHook = TraceHook()
    debugHook.hook()

    # Start tracing when entry point is hit
    ep = ida_ida.inf_get_start_ip()
    ida_dbg.enable_step_trace(1)
    ida_dbg.set_step_trace_options(ida_dbg.ST_OVER_DEBUG_SEG | ida_dbg.ST_OVER_LIB_FUNC)
    print("Running to %x" % ep)
    ida_dbg.run_to(ep)

    while ida_dbg.get_process_state() != 0:
        ida_dbg.wait_for_next_event(1, 0)

    if not debugHook.epReached:
        raise Exception("Entry point wasn't reached!")

    if not debugHook.unhook():
        raise Exception("Error uninstalling hooks!")

    del debugHook

    if then_quit_ida:
        # we're done; exit IDA
        ida_pro.qexit(0)


# load the debugger module depending on the file type
if ida_ida.inf_get_filetype() == ida_ida.f_PE:
    ida_dbg.load_debugger("win32", 0)
elif ida_ida.inf_get_filetype() == ida_ida.f_ELF:
    ida_dbg.load_debugger("linux", 0)
elif ida_ida.inf_get_filetype() == ida_ida.f_MACHO:
    ida_dbg.load_debugger("mac", 0)
if not under_test:
    do_trace()



"""
Routines to manipulate function stack frames, stack variables, register
variables and local labels.

The frame is represented as a structure:
+------------------------------------------------+
  | function arguments                             |
  +------------------------------------------------+
  | return address (isn't stored in func_t)        |
  +------------------------------------------------+
  | saved registers (SI, DI, etc - func_t::frregs) |
  +------------------------------------------------+ <- typical BP
  |                                                |  |
  |                                                |  | func_t::fpd
  |                                                |  |
  |                                                | <- real BP
  | local variables (func_t::frsize)               |
  |                                                |
  |                                                |
  +------------------------------------------------+ <- SP

To access the structure of a function frame, use:
* get_struc() (use func_t::frame as structure ID)
* get_frame(const func_t *pfn)
* get_frame(ea_t ea)"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _ida_frame
else:
    import _ida_frame

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

SWIG_PYTHON_LEGACY_BOOL = _ida_frame.SWIG_PYTHON_LEGACY_BOOL

import ida_idaapi

import ida_range
class xreflist_t(object):
    r"""
    Proxy of C++ qvector< xreflist_entry_t > class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(self) -> xreflist_t
        __init__(self, x) -> xreflist_t

        @param x: qvector< xreflist_entry_t > const &
        """
        _ida_frame.xreflist_t_swiginit(self, _ida_frame.new_xreflist_t(*args))
    __swig_destroy__ = _ida_frame.delete_xreflist_t

    def push_back(self, *args) -> "xreflist_entry_t &":
        r"""
        push_back(self, x)

        @param x: xreflist_entry_t const &

        push_back(self) -> xreflist_entry_t
        """
        return _ida_frame.xreflist_t_push_back(self, *args)

    def pop_back(self, *args) -> "void":
        r"""
        pop_back(self)
        """
        return _ida_frame.xreflist_t_pop_back(self, *args)

    def size(self, *args) -> "size_t":
        r"""
        size(self) -> size_t
        """
        return _ida_frame.xreflist_t_size(self, *args)

    def empty(self, *args) -> "bool":
        r"""
        empty(self) -> bool
        """
        return _ida_frame.xreflist_t_empty(self, *args)

    def at(self, *args) -> "xreflist_entry_t const &":
        r"""
        at(self, _idx) -> xreflist_entry_t

        @param _idx: size_t
        """
        return _ida_frame.xreflist_t_at(self, *args)

    def qclear(self, *args) -> "void":
        r"""
        qclear(self)
        """
        return _ida_frame.xreflist_t_qclear(self, *args)

    def clear(self, *args) -> "void":
        r"""
        clear(self)
        """
        return _ida_frame.xreflist_t_clear(self, *args)

    def resize(self, *args) -> "void":
        r"""
        resize(self, _newsize, x)

        @param _newsize: size_t
        @param x: xreflist_entry_t const &

        resize(self, _newsize)

        @param _newsize: size_t
        """
        return _ida_frame.xreflist_t_resize(self, *args)

    def grow(self, *args) -> "void":
        r"""
        grow(self, x=xreflist_entry_t())

        @param x: xreflist_entry_t const &
        """
        return _ida_frame.xreflist_t_grow(self, *args)

    def capacity(self, *args) -> "size_t":
        r"""
        capacity(self) -> size_t
        """
        return _ida_frame.xreflist_t_capacity(self, *args)

    def reserve(self, *args) -> "void":
        r"""
        reserve(self, cnt)

        @param cnt: size_t
        """
        return _ida_frame.xreflist_t_reserve(self, *args)

    def truncate(self, *args) -> "void":
        r"""
        truncate(self)
        """
        return _ida_frame.xreflist_t_truncate(self, *args)

    def swap(self, *args) -> "void":
        r"""
        swap(self, r)

        @param r: qvector< xreflist_entry_t > &
        """
        return _ida_frame.xreflist_t_swap(self, *args)

    def extract(self, *args) -> "xreflist_entry_t *":
        r"""
        extract(self) -> xreflist_entry_t
        """
        return _ida_frame.xreflist_t_extract(self, *args)

    def inject(self, *args) -> "void":
        r"""
        inject(self, s, len)

        @param s: xreflist_entry_t *
        @param len: size_t
        """
        return _ida_frame.xreflist_t_inject(self, *args)

    def __eq__(self, *args) -> "bool":
        r"""
        __eq__(self, r) -> bool

        @param r: qvector< xreflist_entry_t > const &
        """
        return _ida_frame.xreflist_t___eq__(self, *args)

    def __ne__(self, *args) -> "bool":
        r"""
        __ne__(self, r) -> bool

        @param r: qvector< xreflist_entry_t > const &
        """
        return _ida_frame.xreflist_t___ne__(self, *args)

    def begin(self, *args) -> "qvector< xreflist_entry_t >::const_iterator":
        r"""
        begin(self) -> xreflist_entry_t
        """
        return _ida_frame.xreflist_t_begin(self, *args)

    def end(self, *args) -> "qvector< xreflist_entry_t >::const_iterator":
        r"""
        end(self) -> xreflist_entry_t
        """
        return _ida_frame.xreflist_t_end(self, *args)

    def insert(self, *args) -> "qvector< xreflist_entry_t >::iterator":
        r"""
        insert(self, it, x) -> xreflist_entry_t

        @param it: qvector< xreflist_entry_t >::iterator
        @param x: xreflist_entry_t const &
        """
        return _ida_frame.xreflist_t_insert(self, *args)

    def erase(self, *args) -> "qvector< xreflist_entry_t >::iterator":
        r"""
        erase(self, it) -> xreflist_entry_t

        @param it: qvector< xreflist_entry_t >::iterator

        erase(self, first, last) -> xreflist_entry_t

        @param first: qvector< xreflist_entry_t >::iterator
        @param last: qvector< xreflist_entry_t >::iterator
        """
        return _ida_frame.xreflist_t_erase(self, *args)

    def find(self, *args) -> "qvector< xreflist_entry_t >::const_iterator":
        r"""
        find(self, x) -> xreflist_entry_t

        @param x: xreflist_entry_t const &

        """
        return _ida_frame.xreflist_t_find(self, *args)

    def has(self, *args) -> "bool":
        r"""
        has(self, x) -> bool

        @param x: xreflist_entry_t const &
        """
        return _ida_frame.xreflist_t_has(self, *args)

    def add_unique(self, *args) -> "bool":
        r"""
        add_unique(self, x) -> bool

        @param x: xreflist_entry_t const &
        """
        return _ida_frame.xreflist_t_add_unique(self, *args)

    def _del(self, *args) -> "bool":
        r"""
        _del(self, x) -> bool

        Parameters
        ----------
        x: xreflist_entry_t const &

        """
        return _ida_frame.xreflist_t__del(self, *args)

    def __len__(self, *args) -> "size_t":
        r"""
        __len__(self) -> size_t
        """
        return _ida_frame.xreflist_t___len__(self, *args)

    def __getitem__(self, *args) -> "xreflist_entry_t const &":
        r"""
        __getitem__(self, i) -> xreflist_entry_t

        @param i: size_t
        """
        return _ida_frame.xreflist_t___getitem__(self, *args)

    def __setitem__(self, *args) -> "void":
        r"""
        __setitem__(self, i, v)

        @param i: size_t
        @param v: xreflist_entry_t const &
        """
        return _ida_frame.xreflist_t___setitem__(self, *args)

    front = ida_idaapi._qvector_front
    back = ida_idaapi._qvector_back
    __iter__ = ida_idaapi._bounded_getitem_iterator


# Register xreflist_t in _ida_frame:
_ida_frame.xreflist_t_swigregister(xreflist_t)


def get_stkvar(*args) -> "PyObject *":
    r"""
    get_stkvar(insn, op, v) -> (member_t, int) or None
    Get pointer to stack variable

    @param insn: an ida_ua.insn_t, or an address (C++: const insn_t &)
    @param op: reference to instruction operand
    @param v: immediate value in the operand (usually op.addr)
    @return:     - None on failure
        - tuple(member_t, actval)
          where actval: actual value used to fetch stack variable
    """
    return _ida_frame.get_stkvar(*args)

def is_funcarg_off(*args) -> "bool":
    r"""
    is_funcarg_off(pfn, frameoff) -> bool

    @param pfn: func_t const *
    @param frameoff: uval_t
    """
    return _ida_frame.is_funcarg_off(*args)

def lvar_off(*args) -> "sval_t":
    r"""
    lvar_off(pfn, frameoff) -> sval_t

    @param pfn: func_t const *
    @param frameoff: uval_t
    """
    return _ida_frame.lvar_off(*args)
class stkpnt_t(object):
    r"""
    Proxy of C++ stkpnt_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ea = property(_ida_frame.stkpnt_t_ea_get, _ida_frame.stkpnt_t_ea_set, doc=r"""ea""")
    spd = property(_ida_frame.stkpnt_t_spd_get, _ida_frame.stkpnt_t_spd_set, doc=r"""spd""")

    def __eq__(self, *args) -> "bool":
        r"""
        __eq__(self, r) -> bool

        @param r: stkpnt_t const &
        """
        return _ida_frame.stkpnt_t___eq__(self, *args)

    def __ne__(self, *args) -> "bool":
        r"""
        __ne__(self, r) -> bool

        @param r: stkpnt_t const &
        """
        return _ida_frame.stkpnt_t___ne__(self, *args)

    def __lt__(self, *args) -> "bool":
        r"""
        __lt__(self, r) -> bool

        @param r: stkpnt_t const &
        """
        return _ida_frame.stkpnt_t___lt__(self, *args)

    def __gt__(self, *args) -> "bool":
        r"""
        __gt__(self, r) -> bool

        @param r: stkpnt_t const &
        """
        return _ida_frame.stkpnt_t___gt__(self, *args)

    def __le__(self, *args) -> "bool":
        r"""
        __le__(self, r) -> bool

        @param r: stkpnt_t const &
        """
        return _ida_frame.stkpnt_t___le__(self, *args)

    def __ge__(self, *args) -> "bool":
        r"""
        __ge__(self, r) -> bool

        @param r: stkpnt_t const &
        """
        return _ida_frame.stkpnt_t___ge__(self, *args)

    def compare(self, *args) -> "int":
        r"""
        compare(self, r) -> int

        @param r: stkpnt_t const &
        """
        return _ida_frame.stkpnt_t_compare(self, *args)

    def __init__(self, *args):
        r"""
        __init__(self) -> stkpnt_t
        """
        _ida_frame.stkpnt_t_swiginit(self, _ida_frame.new_stkpnt_t(*args))
    __swig_destroy__ = _ida_frame.delete_stkpnt_t

# Register stkpnt_t in _ida_frame:
_ida_frame.stkpnt_t_swigregister(stkpnt_t)

class stkpnts_t(object):
    r"""
    Proxy of C++ stkpnts_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __eq__(self, *args) -> "bool":
        r"""
        __eq__(self, r) -> bool

        @param r: stkpnts_t const &
        """
        return _ida_frame.stkpnts_t___eq__(self, *args)

    def __ne__(self, *args) -> "bool":
        r"""
        __ne__(self, r) -> bool

        @param r: stkpnts_t const &
        """
        return _ida_frame.stkpnts_t___ne__(self, *args)

    def __lt__(self, *args) -> "bool":
        r"""
        __lt__(self, r) -> bool

        @param r: stkpnts_t const &
        """
        return _ida_frame.stkpnts_t___lt__(self, *args)

    def __gt__(self, *args) -> "bool":
        r"""
        __gt__(self, r) -> bool

        @param r: stkpnts_t const &
        """
        return _ida_frame.stkpnts_t___gt__(self, *args)

    def __le__(self, *args) -> "bool":
        r"""
        __le__(self, r) -> bool

        @param r: stkpnts_t const &
        """
        return _ida_frame.stkpnts_t___le__(self, *args)

    def __ge__(self, *args) -> "bool":
        r"""
        __ge__(self, r) -> bool

        @param r: stkpnts_t const &
        """
        return _ida_frame.stkpnts_t___ge__(self, *args)

    def compare(self, *args) -> "int":
        r"""
        compare(self, r) -> int

        @param r: stkpnts_t const &
        """
        return _ida_frame.stkpnts_t_compare(self, *args)

    def __init__(self, *args):
        r"""
        __init__(self) -> stkpnts_t
        """
        _ida_frame.stkpnts_t_swiginit(self, _ida_frame.new_stkpnts_t(*args))
    __swig_destroy__ = _ida_frame.delete_stkpnts_t

# Register stkpnts_t in _ida_frame:
_ida_frame.stkpnts_t_swigregister(stkpnts_t)


def add_frame(*args) -> "bool":
    r"""
    add_frame(pfn, frsize, frregs, argsize) -> bool
    Add function frame.

    @param pfn: (C++: func_t *) pointer to function structure
    @param frsize: (C++: sval_t) size of function local variables
    @param frregs: (C++: ushort) size of saved registers
    @param argsize: (C++: asize_t) size of function arguments range which will be purged upon
                    return. this parameter is used for __stdcall and __pascal
                    calling conventions. for other calling conventions please pass
                    0.
    @retval 1: ok
    @retval 0: failed (no function, frame already exists)
    """
    return _ida_frame.add_frame(*args)

def del_frame(*args) -> "bool":
    r"""
    del_frame(pfn) -> bool
    Delete a function frame.

    @param pfn: (C++: func_t *) pointer to function structure
    @return: success
    """
    return _ida_frame.del_frame(*args)

def set_frame_size(*args) -> "bool":
    r"""
    set_frame_size(pfn, frsize, frregs, argsize) -> bool
    Set size of function frame. Note: The returned size may not include all stack
    arguments. It does so only for __stdcall and __fastcall calling conventions. To
    get the entire frame size for all cases use get_struc_size(get_frame(pfn)).

    @param pfn: (C++: func_t *) pointer to function structure
    @param frsize: (C++: asize_t) size of function local variables
    @param frregs: (C++: ushort) size of saved registers
    @param argsize: (C++: asize_t) size of function arguments that will be purged from the stack
                    upon return
    @return: success
    """
    return _ida_frame.set_frame_size(*args)

def get_frame_size(*args) -> "asize_t":
    r"""
    get_frame_size(pfn) -> asize_t
    Get full size of a function frame. This function takes into account size of
    local variables + size of saved registers + size of return address + number of
    purged bytes. The purged bytes correspond to the arguments of the functions with
    __stdcall and __fastcall calling conventions.

    @param pfn: (C++: const func_t *) pointer to function structure, may be nullptr
    @return: size of frame in bytes or zero
    """
    return _ida_frame.get_frame_size(*args)

def get_frame_retsize(*args) -> "int":
    r"""
    get_frame_retsize(pfn) -> int
    Get size of function return address.

    @param pfn: (C++: const func_t *) pointer to function structure, can't be nullptr
    """
    return _ida_frame.get_frame_retsize(*args)
FPC_ARGS = _ida_frame.FPC_ARGS

FPC_RETADDR = _ida_frame.FPC_RETADDR

FPC_SAVREGS = _ida_frame.FPC_SAVREGS

FPC_LVARS = _ida_frame.FPC_LVARS


def get_frame_part(*args) -> "void":
    r"""
    get_frame_part(range, pfn, part)
    Get offsets of the frame part in the frame.

    @param range: (C++: range_t *) pointer to the output buffer with the frame part
                  start/end(exclusive) offsets, can't be nullptr
    @param pfn: (C++: const func_t *) pointer to function structure, can't be nullptr
    @param part: (C++: frame_part_t) frame part
    """
    return _ida_frame.get_frame_part(*args)

def frame_off_args(*args) -> "ea_t":
    r"""
    frame_off_args(pfn) -> ea_t
    Get starting address of arguments section.

    @param pfn: (C++: const func_t *) func_t const *
    """
    return _ida_frame.frame_off_args(*args)

def frame_off_retaddr(*args) -> "ea_t":
    r"""
    frame_off_retaddr(pfn) -> ea_t
    Get starting address of return address section.

    @param pfn: (C++: const func_t *) func_t const *
    """
    return _ida_frame.frame_off_retaddr(*args)

def frame_off_savregs(*args) -> "ea_t":
    r"""
    frame_off_savregs(pfn) -> ea_t
    Get starting address of saved registers section.

    @param pfn: (C++: const func_t *) func_t const *
    """
    return _ida_frame.frame_off_savregs(*args)

def frame_off_lvars(*args) -> "ea_t":
    r"""
    frame_off_lvars(pfn) -> ea_t
    Get start address of local variables section.

    @param pfn: (C++: const func_t *) func_t const *
    """
    return _ida_frame.frame_off_lvars(*args)

def get_frame(*args) -> "struc_t *":
    r"""
    get_frame(pfn) -> struc_t *
    Get pointer to function frame.

    @param pfn: func_t const *
    """
    return _ida_frame.get_frame(*args)

def soff_to_fpoff(*args) -> "sval_t":
    r"""
    soff_to_fpoff(pfn, soff) -> sval_t
    Convert struct offsets into fp-relative offsets. This function converts the
    offsets inside the struc_t object into the frame pointer offsets (for example,
    EBP-relative).

    @param pfn: (C++: func_t *)
    @param soff: (C++: uval_t)
    """
    return _ida_frame.soff_to_fpoff(*args)

def update_fpd(*args) -> "bool":
    r"""
    update_fpd(pfn, fpd) -> bool
    Update frame pointer delta.

    @param pfn: (C++: func_t *) pointer to function structure
    @param fpd: (C++: asize_t) new fpd value. cannot be bigger than the local variable range size.
    @return: success
    """
    return _ida_frame.update_fpd(*args)

def set_purged(*args) -> "bool":
    r"""
    set_purged(ea, nbytes, override_old_value) -> bool
    Set the number of purged bytes for a function or data item (funcptr). This
    function will update the database and plan to reanalyze items referencing the
    specified address. It works only for processors with PR_PURGING bit in 16 and 32
    bit modes.

    @param ea: (C++: ea_t) address of the function of item
    @param nbytes: (C++: int) number of purged bytes
    @param override_old_value: (C++: bool) may overwrite old information about purged bytes
    @return: success
    """
    return _ida_frame.set_purged(*args)

def get_func_by_frame(*args) -> "ea_t":
    r"""
    get_func_by_frame(frame_id) -> ea_t
    Get function by its frame id.
    @warning: this function works only with databases created by IDA > 5.6

    @param frame_id: (C++: tid_t) id of the function frame
    @return: start address of the function or BADADDR
    """
    return _ida_frame.get_func_by_frame(*args)
STKVAR_VALID_SIZE = _ida_frame.STKVAR_VALID_SIZE
r"""
x.dtyp contains correct variable type (for insns like 'lea' this bit must be
off). In general, dr_O references do not allow to determine the variable size
"""


def define_stkvar(*args) -> "bool":
    r"""
    define_stkvar(pfn, name, off, flags, ti, nbytes) -> bool
    Define/redefine a stack variable.

    @param pfn: (C++: func_t *) pointer to function
    @param name: (C++: const char *) variable name, nullptr means autogenerate a name
    @param off: (C++: sval_t) offset of the stack variable in the frame. negative values denote
                local variables, positive - function arguments.
    @param flags: (C++: flags64_t) variable type flags (byte_flag() for a byte variable, for example)
    @param ti: (C++: const opinfo_t *) additional type information (like offsets, structs, etc)
    @param nbytes: (C++: asize_t) number of bytes occupied by the variable
    @return: success
    """
    return _ida_frame.define_stkvar(*args)

def build_stkvar_name(*args) -> "qstring *":
    r"""
    build_stkvar_name(pfn, v) -> str
    Build automatic stack variable name.

    @param pfn: (C++: const func_t *) pointer to function (can't be nullptr!)
    @param v: (C++: sval_t) value of variable offset
    @return: length of stack variable name or -1
    """
    return _ida_frame.build_stkvar_name(*args)

def calc_stkvar_struc_offset(*args) -> "ea_t":
    r"""
    calc_stkvar_struc_offset(pfn, insn, n) -> ea_t
    Calculate offset of stack variable in the frame structure.

    @param pfn: (C++: func_t *) pointer to function (can't be nullptr!)
    @param insn: (C++: const insn_t &) the instruction
    @param n: (C++: int) 0..UA_MAXOP-1 operand number -1 if error, return BADADDR
    @return: BADADDR if some error (issue a warning if stack frame is bad)
    """
    return _ida_frame.calc_stkvar_struc_offset(*args)

def free_regvar(*args) -> "void":
    r"""
    free_regvar(v)

    @param v: regvar_t *
    """
    return _ida_frame.free_regvar(*args)
class regvar_t(ida_range.range_t):
    r"""
    Proxy of C++ regvar_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    canon = property(_ida_frame.regvar_t_canon_get, _ida_frame.regvar_t_canon_set, doc=r"""canon""")
    r"""
    canonical register name (case-insensitive)
    """
    user = property(_ida_frame.regvar_t_user_get, _ida_frame.regvar_t_user_set, doc=r"""user""")
    r"""
    user-defined register name
    """
    cmt = property(_ida_frame.regvar_t_cmt_get, _ida_frame.regvar_t_cmt_set, doc=r"""cmt""")
    r"""
    comment to appear near definition
    """

    def __init__(self, *args):
        r"""
        __init__(self) -> regvar_t
        __init__(self, r) -> regvar_t

        @param r: regvar_t const &
        """
        _ida_frame.regvar_t_swiginit(self, _ida_frame.new_regvar_t(*args))
    __swig_destroy__ = _ida_frame.delete_regvar_t

    def swap(self, *args) -> "void":
        r"""
        swap(self, r)

        @param r: regvar_t &
        """
        return _ida_frame.regvar_t_swap(self, *args)

# Register regvar_t in _ida_frame:
_ida_frame.regvar_t_swigregister(regvar_t)


def add_regvar(*args) -> "int":
    r"""
    add_regvar(pfn, ea1, ea2, canon, user, cmt) -> int
    Define a register variable.

    @param pfn: (C++: func_t *) function in which the definition will be created
    @param ea1: (C++: ea_t) ,ea2: range of addresses within the function where the definition will
                    be used
    @param canon: (C++: const char *) name of a general register
    @param canon: (C++: const char *) name of a general register
    @param user: (C++: const char *) user-defined name for the register
    @param cmt: (C++: const char *) comment for the definition
    @return: Register variable error codes
    """
    return _ida_frame.add_regvar(*args)
REGVAR_ERROR_OK = _ida_frame.REGVAR_ERROR_OK
r"""
all ok
"""

REGVAR_ERROR_ARG = _ida_frame.REGVAR_ERROR_ARG
r"""
function arguments are bad
"""

REGVAR_ERROR_RANGE = _ida_frame.REGVAR_ERROR_RANGE
r"""
the definition range is bad
"""

REGVAR_ERROR_NAME = _ida_frame.REGVAR_ERROR_NAME
r"""
the provided name(s) can't be accepted
"""


def find_regvar(*args) -> "regvar_t *":
    r"""
    find_regvar(pfn, ea1, ea2, canon, user) -> regvar_t
    Find a register variable definition.

    @param pfn: (C++: func_t *) function in question
    @param ea1: ea_t
    @param canon: (C++: const char *) name of a general register
    @param canon: (C++: const char *) name of a general register
    @param user: char const *

    @return: nullptr-not found, otherwise ptr to regvar_t
    find_regvar(pfn, ea, canon) -> regvar_t

    @param pfn: func_t *
    @param ea: ea_t
    @param canon: char const *
    """
    return _ida_frame.find_regvar(*args)

def has_regvar(*args) -> "bool":
    r"""
    has_regvar(pfn, ea) -> bool
    Is there a register variable definition?

    @param pfn: (C++: func_t *) function in question
    @param ea: (C++: ea_t) current address
    """
    return _ida_frame.has_regvar(*args)

def rename_regvar(*args) -> "int":
    r"""
    rename_regvar(pfn, v, user) -> int
    Rename a register variable.

    @param pfn: (C++: func_t *) function in question
    @param v: (C++: regvar_t *) variable to rename
    @param user: (C++: const char *) new user-defined name for the register
    @return: Register variable error codes
    """
    return _ida_frame.rename_regvar(*args)

def set_regvar_cmt(*args) -> "int":
    r"""
    set_regvar_cmt(pfn, v, cmt) -> int
    Set comment for a register variable.

    @param pfn: (C++: func_t *) function in question
    @param v: (C++: regvar_t *) variable to rename
    @param cmt: (C++: const char *) new comment
    @return: Register variable error codes
    """
    return _ida_frame.set_regvar_cmt(*args)

def del_regvar(*args) -> "int":
    r"""
    del_regvar(pfn, ea1, ea2, canon) -> int
    Delete a register variable definition.

    @param pfn: (C++: func_t *) function in question
    @param ea1: (C++: ea_t) ,ea2: range of addresses within the function where the definition
                    holds
    @param canon: (C++: const char *) name of a general register
    @param canon: (C++: const char *) name of a general register
    @return: Register variable error codes
    """
    return _ida_frame.del_regvar(*args)

def add_auto_stkpnt(*args) -> "bool":
    r"""
    add_auto_stkpnt(pfn, ea, delta) -> bool
    Add automatic SP register change point.

    @param pfn: (C++: func_t *) pointer to function. may be nullptr.
    @param ea: (C++: ea_t) linear address where SP changes. usually this is the end of the
               instruction which modifies the stack pointer ( insn_t::ea+
               insn_t::size)
    @param delta: (C++: sval_t) difference between old and new values of SP
    @return: success
    """
    return _ida_frame.add_auto_stkpnt(*args)

def add_user_stkpnt(*args) -> "bool":
    r"""
    add_user_stkpnt(ea, delta) -> bool
    Add user-defined SP register change point.

    @param ea: (C++: ea_t) linear address where SP changes
    @param delta: (C++: sval_t) difference between old and new values of SP
    @return: success
    """
    return _ida_frame.add_user_stkpnt(*args)

def del_stkpnt(*args) -> "bool":
    r"""
    del_stkpnt(pfn, ea) -> bool
    Delete SP register change point.

    @param pfn: (C++: func_t *) pointer to function. may be nullptr.
    @param ea: (C++: ea_t) linear address
    @return: success
    """
    return _ida_frame.del_stkpnt(*args)

def get_spd(*args) -> "sval_t":
    r"""
    get_spd(pfn, ea) -> sval_t
    Get difference between the initial and current values of ESP.

    @param pfn: (C++: func_t *) pointer to function. may be nullptr.
    @param ea: (C++: ea_t) linear address of an instruction
    @return: 0 or the difference, usually a negative number. returns the sp-diff
             before executing the instruction.
    """
    return _ida_frame.get_spd(*args)

def get_effective_spd(*args) -> "sval_t":
    r"""
    get_effective_spd(pfn, ea) -> sval_t
    Get effective difference between the initial and current values of ESP. This
    function returns the sp-diff used by the instruction. The difference between
    get_spd() and get_effective_spd() is present only for instructions like "pop
    [esp+N]": they modify sp and use the modified value.

    @param pfn: (C++: func_t *) pointer to function. may be nullptr.
    @param ea: (C++: ea_t) linear address
    @return: 0 or the difference, usually a negative number
    """
    return _ida_frame.get_effective_spd(*args)

def get_sp_delta(*args) -> "sval_t":
    r"""
    get_sp_delta(pfn, ea) -> sval_t
    Get modification of SP made at the specified location

    @param pfn: (C++: func_t *) pointer to function. may be nullptr.
    @param ea: (C++: ea_t) linear address
    @return: 0 if the specified location doesn't contain a SP change point.
             otherwise return delta of SP modification.
    """
    return _ida_frame.get_sp_delta(*args)

def recalc_spd(*args) -> "bool":
    r"""
    recalc_spd(cur_ea) -> bool
    Recalculate SP delta for an instruction that stops execution. The next
    instruction is not reached from the current instruction. We need to recalculate
    SP for the next instruction.

    This function will create a new automatic SP register change point if necessary.
    It should be called from the emulator (emu.cpp) when auto_state == AU_USED if
    the current instruction doesn't pass the execution flow to the next instruction.

    @param cur_ea: (C++: ea_t) linear address of the current instruction
    @retval 1: new stkpnt is added
    @retval 0: nothing is changed
    """
    return _ida_frame.recalc_spd(*args)
class xreflist_entry_t(object):
    r"""
    Proxy of C++ xreflist_entry_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ea = property(_ida_frame.xreflist_entry_t_ea_get, _ida_frame.xreflist_entry_t_ea_set, doc=r"""ea""")
    r"""
    Location of the insn referencing the stack frame member.
    """
    opnum = property(_ida_frame.xreflist_entry_t_opnum_get, _ida_frame.xreflist_entry_t_opnum_set, doc=r"""opnum""")
    r"""
    Number of the operand of that instruction.
    """
    type = property(_ida_frame.xreflist_entry_t_type_get, _ida_frame.xreflist_entry_t_type_set, doc=r"""type""")
    r"""
    The type of xref (cref_t & dref_t)
    """

    def __eq__(self, *args) -> "bool":
        r"""
        __eq__(self, r) -> bool

        @param r: xreflist_entry_t const &
        """
        return _ida_frame.xreflist_entry_t___eq__(self, *args)

    def __ne__(self, *args) -> "bool":
        r"""
        __ne__(self, r) -> bool

        @param r: xreflist_entry_t const &
        """
        return _ida_frame.xreflist_entry_t___ne__(self, *args)

    def __lt__(self, *args) -> "bool":
        r"""
        __lt__(self, r) -> bool

        @param r: xreflist_entry_t const &
        """
        return _ida_frame.xreflist_entry_t___lt__(self, *args)

    def __gt__(self, *args) -> "bool":
        r"""
        __gt__(self, r) -> bool

        @param r: xreflist_entry_t const &
        """
        return _ida_frame.xreflist_entry_t___gt__(self, *args)

    def __le__(self, *args) -> "bool":
        r"""
        __le__(self, r) -> bool

        @param r: xreflist_entry_t const &
        """
        return _ida_frame.xreflist_entry_t___le__(self, *args)

    def __ge__(self, *args) -> "bool":
        r"""
        __ge__(self, r) -> bool

        @param r: xreflist_entry_t const &
        """
        return _ida_frame.xreflist_entry_t___ge__(self, *args)

    def compare(self, *args) -> "int":
        r"""
        compare(self, r) -> int

        @param r: xreflist_entry_t const &
        """
        return _ida_frame.xreflist_entry_t_compare(self, *args)

    def __init__(self, *args):
        r"""
        __init__(self) -> xreflist_entry_t
        """
        _ida_frame.xreflist_entry_t_swiginit(self, _ida_frame.new_xreflist_entry_t(*args))
    __swig_destroy__ = _ida_frame.delete_xreflist_entry_t

# Register xreflist_entry_t in _ida_frame:
_ida_frame.xreflist_entry_t_swigregister(xreflist_entry_t)


def build_stkvar_xrefs(*args) -> "void":
    r"""
    build_stkvar_xrefs(out, pfn, mptr)
    Fill 'out' with a list of all the xrefs made from function 'pfn', to the
    argument or variable 'mptr' in 'pfn's stack frame.

    @param out: (C++: xreflist_t *) the list of xrefs to fill.
    @param pfn: (C++: func_t *) the function to scan.
    @param mptr: (C++: const member_t *) the argument/variable in pfn's stack frame.
    """
    return _ida_frame.build_stkvar_xrefs(*args)

def get_min_spd_ea(*args) -> "ea_t":
    r"""
    get_min_spd_ea(pfn) -> ea_t

    @param pfn: func_t *
    """
    return _ida_frame.get_min_spd_ea(*args)

def delete_unreferenced_stkvars(*args) -> "int":
    r"""
    delete_unreferenced_stkvars(pfn) -> int

    @param pfn: func_t *
    """
    return _ida_frame.delete_unreferenced_stkvars(*args)

def delete_wrong_stkvar_ops(*args) -> "int":
    r"""
    delete_wrong_stkvar_ops(pfn) -> int

    @param pfn: func_t *
    """
    return _ida_frame.delete_wrong_stkvar_ops(*args)






"""
Definitions of IDP, LDR, PLUGIN module interfaces.

This file also contains:
* functions to load files into the database
* functions to generate output files
* high level functions to work with the database (open, save, close)

The LDR interface consists of one structure: loader_t
The IDP interface consists of one structure: processor_t
The PLUGIN interface consists of one structure: plugin_t

Modules can't use standard FILE* functions. They must use functions from
<fpro.h>

Modules can't use standard memory allocation functions. They must use functions
from <pro.h>

The exported entry #1 in the module should point to the the appropriate
structure. (loader_t for LDR module, for example)"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _ida_loader
else:
    import _ida_loader

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

SWIG_PYTHON_LEGACY_BOOL = _ida_loader.SWIG_PYTHON_LEGACY_BOOL

import ida_idaapi

class qvector_snapshotvec_t(object):
    r"""
    Proxy of C++ qvector< snapshot_t * > class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(self) -> qvector_snapshotvec_t
        __init__(self, x) -> qvector_snapshotvec_t

        @param x: qvector< snapshot_t * > const &
        """
        _ida_loader.qvector_snapshotvec_t_swiginit(self, _ida_loader.new_qvector_snapshotvec_t(*args))
    __swig_destroy__ = _ida_loader.delete_qvector_snapshotvec_t

    def push_back(self, *args) -> "snapshot_t *&":
        r"""
        push_back(self, x)

        @param x: snapshot_t *const &

        push_back(self) -> snapshot_t *&
        """
        return _ida_loader.qvector_snapshotvec_t_push_back(self, *args)

    def pop_back(self, *args) -> "void":
        r"""
        pop_back(self)
        """
        return _ida_loader.qvector_snapshotvec_t_pop_back(self, *args)

    def size(self, *args) -> "size_t":
        r"""
        size(self) -> size_t
        """
        return _ida_loader.qvector_snapshotvec_t_size(self, *args)

    def empty(self, *args) -> "bool":
        r"""
        empty(self) -> bool
        """
        return _ida_loader.qvector_snapshotvec_t_empty(self, *args)

    def at(self, *args) -> "snapshot_t *const &":
        r"""
        at(self, _idx) -> snapshot_t

        @param _idx: size_t
        """
        return _ida_loader.qvector_snapshotvec_t_at(self, *args)

    def qclear(self, *args) -> "void":
        r"""
        qclear(self)
        """
        return _ida_loader.qvector_snapshotvec_t_qclear(self, *args)

    def clear(self, *args) -> "void":
        r"""
        clear(self)
        """
        return _ida_loader.qvector_snapshotvec_t_clear(self, *args)

    def resize(self, *args) -> "void":
        r"""
        resize(self, _newsize, x)

        @param _newsize: size_t
        @param x: snapshot_t *const &

        resize(self, _newsize)

        @param _newsize: size_t
        """
        return _ida_loader.qvector_snapshotvec_t_resize(self, *args)

    def capacity(self, *args) -> "size_t":
        r"""
        capacity(self) -> size_t
        """
        return _ida_loader.qvector_snapshotvec_t_capacity(self, *args)

    def reserve(self, *args) -> "void":
        r"""
        reserve(self, cnt)

        @param cnt: size_t
        """
        return _ida_loader.qvector_snapshotvec_t_reserve(self, *args)

    def truncate(self, *args) -> "void":
        r"""
        truncate(self)
        """
        return _ida_loader.qvector_snapshotvec_t_truncate(self, *args)

    def swap(self, *args) -> "void":
        r"""
        swap(self, r)

        @param r: qvector< snapshot_t * > &
        """
        return _ida_loader.qvector_snapshotvec_t_swap(self, *args)

    def extract(self, *args) -> "snapshot_t **":
        r"""
        extract(self) -> snapshot_t **
        """
        return _ida_loader.qvector_snapshotvec_t_extract(self, *args)

    def inject(self, *args) -> "void":
        r"""
        inject(self, s, len)

        @param s: snapshot_t **
        @param len: size_t
        """
        return _ida_loader.qvector_snapshotvec_t_inject(self, *args)

    def __eq__(self, *args) -> "bool":
        r"""
        __eq__(self, r) -> bool

        @param r: qvector< snapshot_t * > const &
        """
        return _ida_loader.qvector_snapshotvec_t___eq__(self, *args)

    def __ne__(self, *args) -> "bool":
        r"""
        __ne__(self, r) -> bool

        @param r: qvector< snapshot_t * > const &
        """
        return _ida_loader.qvector_snapshotvec_t___ne__(self, *args)

    def begin(self, *args) -> "qvector< snapshot_t * >::const_iterator":
        r"""
        begin(self) -> qvector< snapshot_t * >::iterator
        begin(self) -> qvector< snapshot_t * >::const_iterator
        """
        return _ida_loader.qvector_snapshotvec_t_begin(self, *args)

    def end(self, *args) -> "qvector< snapshot_t * >::const_iterator":
        r"""
        end(self) -> qvector< snapshot_t * >::iterator
        end(self) -> qvector< snapshot_t * >::const_iterator
        """
        return _ida_loader.qvector_snapshotvec_t_end(self, *args)

    def insert(self, *args) -> "qvector< snapshot_t * >::iterator":
        r"""
        insert(self, it, x) -> qvector< snapshot_t * >::iterator

        @param it: qvector< snapshot_t * >::iterator
        @param x: snapshot_t *const &
        """
        return _ida_loader.qvector_snapshotvec_t_insert(self, *args)

    def erase(self, *args) -> "qvector< snapshot_t * >::iterator":
        r"""
        erase(self, it) -> qvector< snapshot_t * >::iterator

        @param it: qvector< snapshot_t * >::iterator

        erase(self, first, last) -> qvector< snapshot_t * >::iterator

        @param first: qvector< snapshot_t * >::iterator
        @param last: qvector< snapshot_t * >::iterator
        """
        return _ida_loader.qvector_snapshotvec_t_erase(self, *args)

    def find(self, *args) -> "qvector< snapshot_t * >::const_iterator":
        r"""
        find(self, x) -> qvector< snapshot_t * >::iterator

        @param x: snapshot_t *const &

        find(self, x) -> qvector< snapshot_t * >::const_iterator

        @param x: snapshot_t *const &
        """
        return _ida_loader.qvector_snapshotvec_t_find(self, *args)

    def has(self, *args) -> "bool":
        r"""
        has(self, x) -> bool

        @param x: snapshot_t *const &
        """
        return _ida_loader.qvector_snapshotvec_t_has(self, *args)

    def add_unique(self, *args) -> "bool":
        r"""
        add_unique(self, x) -> bool

        @param x: snapshot_t *const &
        """
        return _ida_loader.qvector_snapshotvec_t_add_unique(self, *args)

    def _del(self, *args) -> "bool":
        r"""
        _del(self, x) -> bool

        Parameters
        ----------
        x: snapshot_t *const &

        """
        return _ida_loader.qvector_snapshotvec_t__del(self, *args)

    def __len__(self, *args) -> "size_t":
        r"""
        __len__(self) -> size_t
        """
        return _ida_loader.qvector_snapshotvec_t___len__(self, *args)

    def __getitem__(self, *args) -> "snapshot_t *const &":
        r"""
        __getitem__(self, i) -> snapshot_t

        @param i: size_t
        """
        return _ida_loader.qvector_snapshotvec_t___getitem__(self, *args)

    def __setitem__(self, *args) -> "void":
        r"""
        __setitem__(self, i, v)

        @param i: size_t
        @param v: snapshot_t *const &
        """
        return _ida_loader.qvector_snapshotvec_t___setitem__(self, *args)

    front = ida_idaapi._qvector_front
    back = ida_idaapi._qvector_back
    __iter__ = ida_idaapi._bounded_getitem_iterator


# Register qvector_snapshotvec_t in _ida_loader:
_ida_loader.qvector_snapshotvec_t_swigregister(qvector_snapshotvec_t)

class loader_t(object):
    r"""
    Proxy of C++ loader_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    version = property(_ida_loader.loader_t_version_get, _ida_loader.loader_t_version_set, doc=r"""version""")
    r"""
    api version, should be IDP_INTERFACE_VERSION
    """
    flags = property(_ida_loader.loader_t_flags_get, _ida_loader.loader_t_flags_set, doc=r"""flags""")
    r"""
    Loader flags
    """

    def __init__(self, *args):
        r"""
        __init__(self) -> loader_t
        """
        _ida_loader.loader_t_swiginit(self, _ida_loader.new_loader_t(*args))
    __swig_destroy__ = _ida_loader.delete_loader_t

# Register loader_t in _ida_loader:
_ida_loader.loader_t_swigregister(loader_t)
LDRF_RELOAD = _ida_loader.LDRF_RELOAD
r"""
loader recognizes NEF_RELOAD flag
"""

LDRF_REQ_PROC = _ida_loader.LDRF_REQ_PROC
r"""
Requires a processor to be set. if this bit is not set, load_file() must call
set_processor_type(..., SETPROC_LOADER)
"""

ACCEPT_ARCHIVE = _ida_loader.ACCEPT_ARCHIVE
r"""
Specify that a file format is served by archive loader See loader_t::accept_file
"""

ACCEPT_CONTINUE = _ida_loader.ACCEPT_CONTINUE
r"""
Specify that the function must be called another time See loader_t::accept_file
"""

ACCEPT_FIRST = _ida_loader.ACCEPT_FIRST
r"""
Specify that a file format should be place first in "load file" dialog box. See
loader_t::accept_file
"""

NEF_SEGS = _ida_loader.NEF_SEGS
r"""
Create segments.
"""

NEF_RSCS = _ida_loader.NEF_RSCS
r"""
Load resources.
"""

NEF_NAME = _ida_loader.NEF_NAME
r"""
Rename entries.
"""

NEF_MAN = _ida_loader.NEF_MAN
r"""
Manual load.
"""

NEF_FILL = _ida_loader.NEF_FILL
r"""
Fill segment gaps.
"""

NEF_IMPS = _ida_loader.NEF_IMPS
r"""
Create import segment.
"""

NEF_FIRST = _ida_loader.NEF_FIRST
r"""
This is the first file loaded into the database.
"""

NEF_CODE = _ida_loader.NEF_CODE
r"""
for load_binary_file(): load as a code segment
"""

NEF_RELOAD = _ida_loader.NEF_RELOAD
r"""
reload the file at the same place:
* don't create segments
* don't create fixup info
* don't import segments
* etc.

Load only the bytes into the base. A loader should have the LDRF_RELOAD bit set.
"""

NEF_FLAT = _ida_loader.NEF_FLAT
r"""
Autocreate FLAT group (PE)
"""

NEF_MINI = _ida_loader.NEF_MINI
r"""
Create mini database (do not copy segment bytes from the input file; use only
the file header metadata)
"""

NEF_LOPT = _ida_loader.NEF_LOPT
r"""
Display additional loader options dialog.
"""

NEF_LALL = _ida_loader.NEF_LALL
r"""
Load all segments without questions.
"""


DLLEXT = _ida_loader.DLLEXT

LOADER_DLL = _ida_loader.LOADER_DLL


def load_binary_file(*args) -> "bool":
    r"""
    load_binary_file(filename, li, _neflags, fileoff, basepara, binoff, nbytes) -> bool
    Load a binary file into the database. This function usually is called from ui.

    @param filename: (C++: const char *) the name of input file as is (if the input file is from
                     library, then this is the name from the library)
    @param li: (C++: linput_t *) loader input source
    @param _neflags: (C++: ushort) Load file flags. For the first file, the flag NEF_FIRST must be
                     set.
    @param fileoff: (C++: qoff64_t) Offset in the input file
    @param basepara: (C++: ea_t) Load address in paragraphs
    @param binoff: (C++: ea_t) Load offset (load_address=(basepara<<4)+binoff)
    @param nbytes: (C++: uint64) Number of bytes to load from the file.
    * 0: up to the end of the file
    @retval true: ok
    @retval false: failed (couldn't open the file)
    """
    return _ida_loader.load_binary_file(*args)

def process_archive(*args) -> "qstring *":
    r"""
    process_archive(temp_file, li, module_name, neflags, defmember, loader) -> str
    Calls loader_t::process_archive() For parameters and return value description
    look at loader_t::process_archive(). Additional parameter 'loader' is a pointer
    to load_info_t structure.

    @param temp_file: (C++: qstring *)
    @param li: (C++: linput_t *)
    @param module_name: (C++: qstring *)
    @param neflags: (C++: ushort *)
    @param defmember: (C++: const char *) char const *
    @param loader: (C++: const load_info_t *) load_info_t const *
    """
    return _ida_loader.process_archive(*args)
OFILE_MAP = _ida_loader.OFILE_MAP
r"""
MAP file.
"""

OFILE_EXE = _ida_loader.OFILE_EXE
r"""
Executable file.
"""

OFILE_IDC = _ida_loader.OFILE_IDC
r"""
IDC file.
"""

OFILE_LST = _ida_loader.OFILE_LST
r"""
Disassembly listing.
"""

OFILE_ASM = _ida_loader.OFILE_ASM
r"""
Assembly.
"""

OFILE_DIF = _ida_loader.OFILE_DIF
r"""
Difference.
"""


def gen_file(*args) -> "int":
    r"""
    gen_file(otype, fp, ea1, ea2, flags) -> int
    Generate an output file.

    @param otype: (C++: ofile_type_t) type of output file.
    @param fp: (C++: FILE *) the output file handle
    @param ea1: (C++: ea_t) start address. For some file types this argument is ignored
    @param ea2: (C++: ea_t) end address. For some file types this argument is ignored as usual
                in ida, the end address of the range is not included
    @param flags: (C++: int) Generate file flagsOFILE_EXE:
    @retval 0: can't generate exe file
    @retval 1: ok
    @return: number of the generated lines. -1 if an error occurred
    """
    return _ida_loader.gen_file(*args)
GENFLG_MAPSEG = _ida_loader.GENFLG_MAPSEG

GENFLG_MAPNAME = _ida_loader.GENFLG_MAPNAME

GENFLG_MAPDMNG = _ida_loader.GENFLG_MAPDMNG

GENFLG_MAPLOC = _ida_loader.GENFLG_MAPLOC

GENFLG_IDCTYPE = _ida_loader.GENFLG_IDCTYPE

GENFLG_ASMTYPE = _ida_loader.GENFLG_ASMTYPE
r"""
OFILE_ASM,OFILE_LST: gen information about types too
"""

GENFLG_GENHTML = _ida_loader.GENFLG_GENHTML
r"""
OFILE_ASM,OFILE_LST: generate html (ui_genfile_callback will be used)
"""

GENFLG_ASMINC = _ida_loader.GENFLG_ASMINC
r"""
OFILE_ASM,OFILE_LST: gen information only about types
"""


def file2base(*args) -> "int":
    r"""
    file2base(li, pos, ea1, ea2, patchable) -> int
    Load portion of file into the database. This function will include (ea1..ea2)
    into the addressing space of the program (make it enabled).

    @param li: (C++: linput_t *) pointer of input source
    @param pos: (C++: qoff64_t) position in the file
    @param ea1: (C++: ea_t) ,ea2: range of destination linear addresses
    @param patchable: (C++: int) should the kernel remember correspondence of file offsets to
                      linear addresses.
    @retval 1: ok
    @retval 0: read error, a warning is displayed
    @note: The storage type of the specified range will be changed to STT_VA.
    @param patchable: (C++: int) should the kernel remember correspondence of file offsets to
                      linear addresses.
    @retval 1: ok
    @retval 0: read error, a warning is displayed
    @note: The storage type of the specified range will be changed to STT_VA.
    """
    return _ida_loader.file2base(*args)
FILEREG_PATCHABLE = _ida_loader.FILEREG_PATCHABLE
r"""
means that the input file may be patched (i.e. no compression, no iterated data,
etc)
"""

FILEREG_NOTPATCHABLE = _ida_loader.FILEREG_NOTPATCHABLE
r"""
the data is kept in some encoded form in the file.
"""


def base2file(*args) -> "int":
    r"""
    base2file(fp, pos, ea1, ea2) -> int
    Unload database to a binary file. This function works for wide byte processors
    too.

    @param fp: (C++: FILE *) pointer to file
    @param pos: (C++: qoff64_t) position in the file
    @param ea1: (C++: ea_t) ,ea2: range of source linear addresses
    @param ea2: (C++: ea_t)
    @return: 1-ok(always), write error leads to immediate exit
    """
    return _ida_loader.base2file(*args)

def get_basic_file_type(*args) -> "filetype_t":
    r"""
    get_basic_file_type(li) -> filetype_t
    Get the input file type. This function can recognize libraries and zip files.

    @param li: (C++: linput_t *)
    """
    return _ida_loader.get_basic_file_type(*args)

def get_file_type_name(*args) -> "size_t":
    r"""
    get_file_type_name() -> str
    Get name of the current file type. The current file type is kept in
    idainfo::filetype.

    @return: size of answer, this function always succeeds
    """
    return _ida_loader.get_file_type_name(*args)

def set_import_ordinal(*args) -> "void":
    r"""
    set_import_ordinal(modnode, ea, ord)
    Set information about the ordinal import entry. This function performs
    'modnode.altset(ord, ea2node(ea));'

    @param modnode: (C++: uval_t) node with information about imported entries
    @param ea: (C++: ea_t) linear address of the entry
    @param ord: (C++: uval_t) ordinal number of the entry
    """
    return _ida_loader.set_import_ordinal(*args)

def set_import_name(*args) -> "void":
    r"""
    set_import_name(modnode, ea, name)
    Set information about the named import entry. This function performs
    'modnode.supset_ea(ea, name);'

    @param modnode: (C++: uval_t) node with information about imported entries
    @param ea: (C++: ea_t) linear address of the entry
    @param name: (C++: const char *) name of the entry
    """
    return _ida_loader.set_import_name(*args)

def load_ids_module(*args) -> "int":
    r"""
    load_ids_module(fname) -> int
    Load and apply IDS file. This function loads the specified IDS file and applies
    it to the database. If the program imports functions from a module with the same
    name as the name of the ids file being loaded, then only functions from this
    module will be affected. Otherwise (i.e. when the program does not import a
    module with this name) any function in the program may be affected.

    @param fname: (C++: char *) name of file to apply
    @retval 1: ok
    @retval 0: some error (a message is displayed). if the ids file does not exist,
               no message is displayed
    """
    return _ida_loader.load_ids_module(*args)

def get_plugin_options(*args) -> "char const *":
    r"""
    get_plugin_options(plugin) -> char const *
    Get plugin options from the command line. If the user has specified the options
    in the -Oplugin_name:options format, them this function will return the
    'options' part of it The 'plugin' parameter should denote the plugin name
    Returns nullptr if there we no options specified

    @param plugin: (C++: const char *) char const *
    """
    return _ida_loader.get_plugin_options(*args)
PLUGIN_DLL = _ida_loader.PLUGIN_DLL
r"""
Pattern to find plugin files.
"""

MODULE_ENTRY_LOADER = _ida_loader.MODULE_ENTRY_LOADER

MODULE_ENTRY_PLUGIN = _ida_loader.MODULE_ENTRY_PLUGIN

MODULE_ENTRY_IDP = _ida_loader.MODULE_ENTRY_IDP

class idp_name_t(object):
    r"""
    Proxy of C++ idp_name_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    lname = property(_ida_loader.idp_name_t_lname_get, _ida_loader.idp_name_t_lname_set, doc=r"""lname""")
    r"""
    long processor name
    """
    sname = property(_ida_loader.idp_name_t_sname_get, _ida_loader.idp_name_t_sname_set, doc=r"""sname""")
    r"""
    short processor name
    """
    hidden = property(_ida_loader.idp_name_t_hidden_get, _ida_loader.idp_name_t_hidden_set, doc=r"""hidden""")
    r"""
    is hidden
    """

    def __init__(self, *args):
        r"""
        __init__(self) -> idp_name_t
        """
        _ida_loader.idp_name_t_swiginit(self, _ida_loader.new_idp_name_t(*args))
    __swig_destroy__ = _ida_loader.delete_idp_name_t

# Register idp_name_t in _ida_loader:
_ida_loader.idp_name_t_swigregister(idp_name_t)

class idp_desc_t(object):
    r"""
    Proxy of C++ idp_desc_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    path = property(_ida_loader.idp_desc_t_path_get, _ida_loader.idp_desc_t_path_set, doc=r"""path""")
    r"""
    module file name
    """
    mtime = property(_ida_loader.idp_desc_t_mtime_get, _ida_loader.idp_desc_t_mtime_set, doc=r"""mtime""")
    r"""
    time of last modification
    """
    family = property(_ida_loader.idp_desc_t_family_get, _ida_loader.idp_desc_t_family_set, doc=r"""family""")
    r"""
    processor's family
    """
    names = property(_ida_loader.idp_desc_t_names_get, _ida_loader.idp_desc_t_names_set, doc=r"""names""")
    r"""
    processor names
    """
    is_script = property(_ida_loader.idp_desc_t_is_script_get, _ida_loader.idp_desc_t_is_script_set, doc=r"""is_script""")
    r"""
    the processor module is a script
    """
    checked = property(_ida_loader.idp_desc_t_checked_get, _ida_loader.idp_desc_t_checked_set, doc=r"""checked""")
    r"""
    internal, for cache management
    """

    def __init__(self, *args):
        r"""
        __init__(self) -> idp_desc_t
        """
        _ida_loader.idp_desc_t_swiginit(self, _ida_loader.new_idp_desc_t(*args))
    __swig_destroy__ = _ida_loader.delete_idp_desc_t

# Register idp_desc_t in _ida_loader:
_ida_loader.idp_desc_t_swigregister(idp_desc_t)

IDP_DLL = _ida_loader.IDP_DLL

class plugin_info_t(object):
    r"""
    Proxy of C++ plugin_info_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    next = property(_ida_loader.plugin_info_t_next_get, _ida_loader.plugin_info_t_next_set, doc=r"""next""")
    r"""
    next plugin information
    """
    path = property(_ida_loader.plugin_info_t_path_get, _ida_loader.plugin_info_t_path_set, doc=r"""path""")
    r"""
    full path to the plugin
    """
    org_name = property(_ida_loader.plugin_info_t_org_name_get, _ida_loader.plugin_info_t_org_name_set, doc=r"""org_name""")
    r"""
    original short name of the plugin
    """
    name = property(_ida_loader.plugin_info_t_name_get, _ida_loader.plugin_info_t_name_set, doc=r"""name""")
    r"""
    short name of the plugin it will appear in the menu
    """
    org_hotkey = property(_ida_loader.plugin_info_t_org_hotkey_get, _ida_loader.plugin_info_t_org_hotkey_set, doc=r"""org_hotkey""")
    r"""
    original hotkey to run the plugin
    """
    hotkey = property(_ida_loader.plugin_info_t_hotkey_get, _ida_loader.plugin_info_t_hotkey_set, doc=r"""hotkey""")
    r"""
    current hotkey to run the plugin
    """
    arg = property(_ida_loader.plugin_info_t_arg_get, _ida_loader.plugin_info_t_arg_set, doc=r"""arg""")
    r"""
    argument used to call the plugin
    """
    entry = property(_ida_loader.plugin_info_t_entry_get, _ida_loader.plugin_info_t_entry_set, doc=r"""entry""")
    r"""
    pointer to the plugin if it is already loaded
    """
    dllmem = property(_ida_loader.plugin_info_t_dllmem_get, _ida_loader.plugin_info_t_dllmem_set, doc=r"""dllmem""")
    flags = property(_ida_loader.plugin_info_t_flags_get, _ida_loader.plugin_info_t_flags_set, doc=r"""flags""")
    r"""
    a copy of plugin_t::flags
    """
    comment = property(_ida_loader.plugin_info_t_comment_get, _ida_loader.plugin_info_t_comment_set, doc=r"""comment""")
    r"""
    a copy of plugin_t::comment
    """

    def __init__(self, *args):
        r"""
        __init__(self) -> plugin_info_t
        """
        _ida_loader.plugin_info_t_swiginit(self, _ida_loader.new_plugin_info_t(*args))
    __swig_destroy__ = _ida_loader.delete_plugin_info_t

# Register plugin_info_t in _ida_loader:
_ida_loader.plugin_info_t_swigregister(plugin_info_t)


def find_plugin(*args) -> "plugin_t *":
    r"""
    find_plugin(name, load_if_needed=False) -> plugin_t *
    Find a user-defined plugin and optionally load it.

    @param name: (C++: const char *) short plugin name without path and extension, or absolute path to
                 the file name
    @param load_if_needed: (C++: bool) if the plugin is not present in the memory, try to load
                           it
    @return: pointer to plugin description block
    """
    return _ida_loader.find_plugin(*args)

def get_fileregion_offset(*args) -> "qoff64_t":
    r"""
    get_fileregion_offset(ea) -> qoff64_t
    Get offset in the input file which corresponds to the given ea. If the specified
    ea can't be mapped into the input file offset, return -1.

    @param ea: (C++: ea_t)
    """
    return _ida_loader.get_fileregion_offset(*args)

def get_fileregion_ea(*args) -> "ea_t":
    r"""
    get_fileregion_ea(offset) -> ea_t
    Get linear address which corresponds to the specified input file offset. If
    can't be found, return BADADDR

    @param offset: (C++: qoff64_t)
    """
    return _ida_loader.get_fileregion_ea(*args)

def gen_exe_file(*args) -> "int":
    r"""
    gen_exe_file(fp) -> int
    Generate an exe file (unload the database in binary form).

    @param fp: (C++: FILE *)
    @return: fp the output file handle. if fp == nullptr then return:
    * 1: can generate an executable file
    * 0: can't generate an executable file
    @retval 1: ok
    @retval 0: failed
    """
    return _ida_loader.gen_exe_file(*args)

def reload_file(*args) -> "bool":
    r"""
    reload_file(file, is_remote) -> bool
    Reload the input file. This function reloads the byte values from the input
    file. It doesn't modify the segmentation, names, comments, etc.

    @param file: (C++: const char *) name of the input file. if file == nullptr then returns:
    * 1: can reload the input file
    * 0: can't reload the input file
    @param is_remote: (C++: bool) is the file located on a remote computer with the debugger
                      server?
    @return: success
    """
    return _ida_loader.reload_file(*args)
MAX_DATABASE_DESCRIPTION = _ida_loader.MAX_DATABASE_DESCRIPTION
r"""
Maximum database snapshot description length.
"""

class snapshot_t(object):
    r"""
    Proxy of C++ snapshot_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    id = property(_ida_loader.snapshot_t_id_get, _ida_loader.snapshot_t_id_set, doc=r"""id""")
    r"""
    snapshot ID. This value is computed using qgettimeofday()
    """
    flags = property(_ida_loader.snapshot_t_flags_get, _ida_loader.snapshot_t_flags_set, doc=r"""flags""")
    r"""
    Snapshot flags
    """
    desc = property(_ida_loader.snapshot_t_desc_get, _ida_loader.snapshot_t_desc_set, doc=r"""desc""")
    r"""
    snapshot description
    """
    filename = property(_ida_loader.snapshot_t_filename_get, _ida_loader.snapshot_t_filename_set, doc=r"""filename""")
    r"""
    snapshot file name
    """
    children = property(_ida_loader.snapshot_t_children_get, _ida_loader.snapshot_t_children_set, doc=r"""children""")
    r"""
    snapshot children
    """

    def __eq__(self, *args) -> "bool":
        r"""
        __eq__(self, r) -> bool

        @param r: snapshot_t const &
        """
        return _ida_loader.snapshot_t___eq__(self, *args)

    def __ne__(self, *args) -> "bool":
        r"""
        __ne__(self, r) -> bool

        @param r: snapshot_t const &
        """
        return _ida_loader.snapshot_t___ne__(self, *args)

    def __lt__(self, *args) -> "bool":
        r"""
        __lt__(self, r) -> bool

        @param r: snapshot_t const &
        """
        return _ida_loader.snapshot_t___lt__(self, *args)

    def __gt__(self, *args) -> "bool":
        r"""
        __gt__(self, r) -> bool

        @param r: snapshot_t const &
        """
        return _ida_loader.snapshot_t___gt__(self, *args)

    def __le__(self, *args) -> "bool":
        r"""
        __le__(self, r) -> bool

        @param r: snapshot_t const &
        """
        return _ida_loader.snapshot_t___le__(self, *args)

    def __ge__(self, *args) -> "bool":
        r"""
        __ge__(self, r) -> bool

        @param r: snapshot_t const &
        """
        return _ida_loader.snapshot_t___ge__(self, *args)

    def clear(self, *args) -> "void":
        r"""
        clear(self)
        """
        return _ida_loader.snapshot_t_clear(self, *args)

    def __init__(self, *args):
        r"""
        __init__(self) -> snapshot_t
        """
        _ida_loader.snapshot_t_swiginit(self, _ida_loader.new_snapshot_t(*args))
    __swig_destroy__ = _ida_loader.delete_snapshot_t

# Register snapshot_t in _ida_loader:
_ida_loader.snapshot_t_swigregister(snapshot_t)
SSF_AUTOMATIC = _ida_loader.SSF_AUTOMATIC
r"""
automatic snapshot
"""



def build_snapshot_tree(*args) -> "bool":
    r"""
    build_snapshot_tree(root) -> bool
    Build the snapshot tree.

    @param root: (C++: snapshot_t *) snapshot root that will contain the snapshot tree elements.
    @return: success
    """
    return _ida_loader.build_snapshot_tree(*args)
SSUF_DESC = _ida_loader.SSUF_DESC
r"""
Update the description.
"""

SSUF_PATH = _ida_loader.SSUF_PATH
r"""
Update the path.
"""

SSUF_FLAGS = _ida_loader.SSUF_FLAGS
r"""
Update the flags.
"""


def flush_buffers(*args) -> "int":
    r"""
    flush_buffers() -> int
    Flush buffers to the disk.
    """
    return _ida_loader.flush_buffers(*args)

def is_trusted_idb(*args) -> "bool":
    r"""
    is_trusted_idb() -> bool
    Is the database considered as trusted?
    """
    return _ida_loader.is_trusted_idb(*args)

def save_database(*args) -> "bool":
    r"""
    save_database(outfile, flags, root=None, attr=None) -> bool
    Save current database using a new file name.

    @param outfile: (C++: const char *) output database file name
    @param flags: (C++: uint32) Database flags
    @param root: (C++: const snapshot_t *) optional: snapshot tree root.
    @param attr: (C++: const snapshot_t *) optional: snapshot attributes
    @note: when both root and attr are not nullptr then the snapshot attributes will
           be updated, otherwise the snapshot attributes will be inherited from the
           current database.
    @return: success
    """
    return _ida_loader.save_database(*args)
DBFL_KILL = _ida_loader.DBFL_KILL
r"""
delete unpacked database
"""

DBFL_COMP = _ida_loader.DBFL_COMP
r"""
collect garbage
"""

DBFL_BAK = _ida_loader.DBFL_BAK
r"""
create backup file (if !DBFL_KILL)
"""

DBFL_TEMP = _ida_loader.DBFL_TEMP
r"""
temporary database
"""


def is_database_flag(*args) -> "bool":
    r"""
    is_database_flag(dbfl) -> bool
    Get the current database flag

    @param dbfl: (C++: uint32) flag Database flags
    @return: the state of the flag (set or cleared)
    """
    return _ida_loader.is_database_flag(*args)

def set_database_flag(*args) -> "void":
    r"""
    set_database_flag(dbfl, cnd=True)
    Set or clear database flag

    @param dbfl: (C++: uint32) flag Database flags
    @param cnd: (C++: bool) set if true or clear flag otherwise
    """
    return _ida_loader.set_database_flag(*args)

def clr_database_flag(*args) -> "void":
    r"""
    clr_database_flag(dbfl)

    @param dbfl: uint32
    """
    return _ida_loader.clr_database_flag(*args)
PATH_TYPE_CMD = _ida_loader.PATH_TYPE_CMD
r"""
full path to the file specified in the command line
"""

PATH_TYPE_IDB = _ida_loader.PATH_TYPE_IDB
r"""
full path of IDB file
"""

PATH_TYPE_ID0 = _ida_loader.PATH_TYPE_ID0
r"""
full path of ID0 file
"""


def get_path(*args) -> "char const *":
    r"""
    get_path(pt) -> char const *
    Get the file path

    @param pt: (C++: path_type_t) file path type Types of the file pathes
    @return: file path, never returns nullptr
    """
    return _ida_loader.get_path(*args)

def set_path(*args) -> "void":
    r"""
    set_path(pt, path)
    Set the file path

    @param pt: (C++: path_type_t) file path type Types of the file pathes
    @param path: (C++: const char *) new file path, use nullptr or empty string to clear the file path
    """
    return _ida_loader.set_path(*args)

def get_elf_debug_file_directory(*args) -> "char const *":
    r"""
    get_elf_debug_file_directory() -> char const *
    Get the value of the ELF_DEBUG_FILE_DIRECTORY configuration directive.
    """
    return _ida_loader.get_elf_debug_file_directory(*args)

def mem2base(*args) -> "int":
    r"""
    mem2base(py_mem, ea, fpos=-1) -> int
    Load database from the memory.

    @param py_mem: the buffer
    @param ea: start linear addresses
    @param fpos: position in the input file the data is taken from.
                 if == -1, then no file position correspond to the data.
    @return:     - Returns zero if the passed buffer was not a string
        - Otherwise 1 is returned
    """
    return _ida_loader.mem2base(*args)

def load_plugin(*args) -> "PyObject *":
    r"""
    load_plugin(name) -> PyCapsule or None
    Loads a plugin

    @param name: char const *
    @return:     - None if plugin could not be loaded
        - An opaque object representing the loaded plugin
    """
    return _ida_loader.load_plugin(*args)

def run_plugin(*args) -> "bool":
    r"""
    run_plugin(plg, arg) -> bool
    Runs a plugin

    @param plg: A plugin object (returned by load_plugin())
    @param arg: size_t
    @return: Boolean
    """
    return _ida_loader.run_plugin(*args)

def load_and_run_plugin(*args) -> "bool":
    r"""
    load_and_run_plugin(name, arg) -> bool
    Load & run a plugin.

    @param name: (C++: const char *) char const *
    @param arg: (C++: size_t)
    """
    return _ida_loader.load_and_run_plugin(*args)

def extract_module_from_archive(*args) -> "PyObject *":
    r"""
    extract_module_from_archive(fname, is_remote=False) -> (NoneType, NoneType), (str, str)
    Extract a module for an archive file. Parse an archive file, show the list of
    modules to the user, allow him to select a module, extract the selected module
    to a file (if the extract module is an archive, repeat the process). This
    function can handle ZIP, AR, AIXAR, OMFLIB files. The temporary file will be
    automatically deleted by IDA at the end.

    @param filename: (C++: char *) in: input file. out: name of the selected module.
    @param is_remote: (C++: bool) is the input file remote?
    @retval true: ok
    @retval false: something bad happened (error message has been displayed to the
                   user)
    """
    return _ida_loader.extract_module_from_archive(*args)






"""
Functions that deal with cross-references.

There are 2 types of xrefs: CODE and DATA references. All xrefs are kept in the
bTree except ordinary execution flow to the next instruction. Ordinary execution
flow to the next instruction is kept in flags (see bytes.hpp)

The source address of a cross-reference must be an item head (is_head) or a
structure member id.

Cross-references are automatically sorted."""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _ida_xref
else:
    import _ida_xref

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

SWIG_PYTHON_LEGACY_BOOL = _ida_xref.SWIG_PYTHON_LEGACY_BOOL

import ida_idaapi


def create_switch_xrefs(*args) -> "bool":
    r"""
    create_switch_xrefs(ea, si) -> bool
    This function creates xrefs from the indirect jump.

    Usually there is no need to call this function directly because the kernel
    will call it for switch tables

    Note: Custom switch information are not supported yet.

    @param ea: address of the 'indirect jump' instruction
    @param si: switch information

    @return: Boolean
    """
    return _ida_xref.create_switch_xrefs(*args)
class cases_and_targets_t(object):
    r"""
    Proxy of C++ cases_and_targets_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    cases = property(_ida_xref.cases_and_targets_t_cases_get, _ida_xref.cases_and_targets_t_cases_set, doc=r"""cases""")
    targets = property(_ida_xref.cases_and_targets_t_targets_get, _ida_xref.cases_and_targets_t_targets_set, doc=r"""targets""")

    def __init__(self, *args):
        r"""
        __init__(self) -> cases_and_targets_t
        """
        _ida_xref.cases_and_targets_t_swiginit(self, _ida_xref.new_cases_and_targets_t(*args))
    __swig_destroy__ = _ida_xref.delete_cases_and_targets_t

# Register cases_and_targets_t in _ida_xref:
_ida_xref.cases_and_targets_t_swigregister(cases_and_targets_t)


def calc_switch_cases(*args) -> "cases_and_targets_t *":
    r"""
    calc_switch_cases(ea, si) -> cases_and_targets_t
    Get information about a switch's cases.

    The returned information can be used as follows:

        for idx in range(len(results.cases)):
            cur_case = results.cases[idx]
            for cidx in range(len(cur_case)):
                print("case: %d" % cur_case[cidx])
            print("  goto 0x%x" % results.targets[idx])

    @param ea: address of the 'indirect jump' instruction
    @param si: switch information

    @return: a structure with 2 members: 'cases', and 'targets'.
    """
    return _ida_xref.calc_switch_cases(*args)

def create_switch_table(*args) -> "bool":
    r"""
    create_switch_table(ea, si) -> bool
    Create switch table from the switch information

    @param ea: address of the 'indirect jump' instruction
    @param si: switch information

    @return: Boolean
    """
    return _ida_xref.create_switch_table(*args)
fl_U = _ida_xref.fl_U
r"""
unknown - for compatibility with old versions. Should not be used anymore.
"""

fl_CF = _ida_xref.fl_CF
r"""
Call Far This xref creates a function at the referenced location
"""

fl_CN = _ida_xref.fl_CN
r"""
Call Near This xref creates a function at the referenced location
"""

fl_JF = _ida_xref.fl_JF
r"""
Jump Far.
"""

fl_JN = _ida_xref.fl_JN
r"""
Jump Near.
"""

fl_USobsolete = _ida_xref.fl_USobsolete
r"""
User specified (obsolete)
"""

fl_F = _ida_xref.fl_F
r"""
Ordinary flow: used to specify execution flow to the next instruction.
"""

dr_U = _ida_xref.dr_U
r"""
Unknown - for compatibility with old versions. Should not be used anymore.
"""

dr_O = _ida_xref.dr_O
r"""
Offset The reference uses 'offset' of data rather than its value OR The
reference appeared because the "OFFSET" flag of instruction is set. The meaning
of this type is IDP dependent.
"""

dr_W = _ida_xref.dr_W
r"""
Write access.
"""

dr_R = _ida_xref.dr_R
r"""
Read access.
"""

dr_T = _ida_xref.dr_T
r"""
Text (for forced operands only) Name of data is used in manual operand
"""

dr_I = _ida_xref.dr_I
r"""
Informational (a derived java class references its base class informationally)
"""

dr_S = _ida_xref.dr_S
r"""
Reference to enum member (symbolic constant)
"""

XREF_USER = _ida_xref.XREF_USER
r"""
User specified xref. This xref will not be deleted by IDA. This bit should be
combined with the existing xref types (cref_t & dref_t) Cannot be used for fl_F
xrefs
"""

XREF_TAIL = _ida_xref.XREF_TAIL
r"""
Reference to tail byte in extrn symbols.
"""

XREF_BASE = _ida_xref.XREF_BASE
r"""
Reference to the base part of an offset.
"""

XREF_MASK = _ida_xref.XREF_MASK
r"""
Mask to get xref type.
"""

XREF_PASTEND = _ida_xref.XREF_PASTEND
r"""
Reference is past item. This bit may be passed to add_dref() functions but it
won't be saved in the database. It will prevent the destruction of eventual
alignment directives.
"""


def xrefchar(*args) -> "char":
    r"""
    xrefchar(xrtype) -> char
    Get character describing the xref type.

    @param xrtype: (C++: char) combination of Cross-Reference type flags and a cref_t of dref_t
                   value
    """
    return _ida_xref.xrefchar(*args)

def add_cref(*args) -> "bool":
    r"""
    add_cref(frm, to, type) -> bool
    Create a code cross-reference.

    @param from: (C++: ea_t) linear address of referencing instruction
    @param to: (C++: ea_t) linear address of referenced instruction
    @param type: (C++: cref_t) cross-reference type
    @return: success
    """
    return _ida_xref.add_cref(*args)

def del_cref(*args) -> "bool":
    r"""
    del_cref(frm, to, expand) -> bool
    Delete a code cross-reference.

    @param from: (C++: ea_t) linear address of referencing instruction
    @param to: (C++: ea_t) linear address of referenced instruction
    @param expand: (C++: bool) policy to delete the referenced instruction
    * 1: plan to delete the referenced instruction if it has no more references.
    * 0: don't delete the referenced instruction even if no more cross-references
    point to it
    @retval true: if the referenced instruction will be deleted
    """
    return _ida_xref.del_cref(*args)

def add_dref(*args) -> "bool":
    r"""
    add_dref(frm, to, type) -> bool
    Create a data cross-reference.

    @param from: (C++: ea_t) linear address of referencing instruction or data
    @param to: (C++: ea_t) linear address of referenced data
    @param type: (C++: dref_t) cross-reference type
    @return: success (may fail if user-defined xref exists from->to)
    """
    return _ida_xref.add_dref(*args)

def del_dref(*args) -> "void":
    r"""
    del_dref(frm, to)
    Delete a data cross-reference.

    @param from: (C++: ea_t) linear address of referencing instruction or data
    @param to: (C++: ea_t) linear address of referenced data
    """
    return _ida_xref.del_dref(*args)
class xrefblk_t(object):
    r"""
    Proxy of C++ xrefblk_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    frm = property(_ida_xref.xrefblk_t_frm_get, _ida_xref.xrefblk_t_frm_set, doc=r"""frm""")
    to = property(_ida_xref.xrefblk_t_to_get, _ida_xref.xrefblk_t_to_set, doc=r"""to""")
    r"""
    the referenced address - filled by first_from(), next_from()
    """
    iscode = property(_ida_xref.xrefblk_t_iscode_get, _ida_xref.xrefblk_t_iscode_set, doc=r"""iscode""")
    r"""
    1-is code reference; 0-is data reference
    """
    type = property(_ida_xref.xrefblk_t_type_get, _ida_xref.xrefblk_t_type_set, doc=r"""type""")
    r"""
    type of the last returned reference (cref_t & dref_t)
    """
    user = property(_ida_xref.xrefblk_t_user_get, _ida_xref.xrefblk_t_user_set, doc=r"""user""")
    r"""
    1-is user defined xref, 0-defined by ida
    """

    def first_from(self, *args) -> "bool":
        r"""
        first_from(self, _from, flags) -> bool
        Get first xref from the given address (store in to)

        @param _from: (C++: ea_t)
        @param flags: (C++: int)
        """
        return _ida_xref.xrefblk_t_first_from(self, *args)

    def first_to(self, *args) -> "bool":
        r"""
        first_to(self, _to, flags) -> bool
        Get xref to given address (store in from)

        @param _to: (C++: ea_t)
        @param flags: (C++: int)
        """
        return _ida_xref.xrefblk_t_first_to(self, *args)

    def next_from(self, *args) -> "bool":
        r"""
        next_from(self) -> bool
        Get xref from '_from' that comes after '_to'.
        next_from(self, _from, _to, flags) -> bool

        @param _from: ea_t
        @param _to: ea_t
        @param flags: int
        """
        return _ida_xref.xrefblk_t_next_from(self, *args)

    def next_to(self, *args) -> "bool":
        r"""
        next_to(self) -> bool
        Get xref to '_to' that comes after '_from'.
        next_to(self, _from, _to, flags) -> bool

        @param _from: ea_t
        @param _to: ea_t
        @param flags: int
        """
        return _ida_xref.xrefblk_t_next_to(self, *args)

    def crefs_to(self, ea):
        r"""
        Provide an iterator on code references to ea including flow references
        """
        ref = get_first_cref_to(ea)
        while ref != ida_idaapi.BADADDR:
            yield ref
            ref = get_next_cref_to(ea, ref)


    def fcrefs_to(self, ea):
        r"""
        Provide an iterator on code references to ea
        """
        ref = get_first_fcref_to(ea)
        while ref != ida_idaapi.BADADDR:
            yield ref
            ref = get_next_fcref_to(ea, ref)


    def crefs_from(self, ea):
        r"""
        Provide an iterator on code references from ea including flow references
        """
        ref = get_first_cref_from(ea)
        while ref != ida_idaapi.BADADDR:
            yield ref
            ref = get_next_cref_from(ea, ref)


    def fcrefs_from(self, ea):
        r"""
        Provide an iterator on code references from ea
        """
        ref = get_first_fcref_from(ea)
        while ref != ida_idaapi.BADADDR:
            yield ref
            ref = get_next_fcref_from(ea, ref)


    def drefs_to(self, ea):
        r"""
        Provide an iterator on data references to ea
        """
        ref = get_first_dref_to(ea)
        while ref != ida_idaapi.BADADDR:
            yield ref
            ref = get_next_dref_to(ea, ref)


    def drefs_from(self, ea):
        r"""
        Provide an iterator on data references from ea
        """
        ref = get_first_dref_from(ea)
        while ref != ida_idaapi.BADADDR:
            yield ref
            ref = get_next_dref_from(ea, ref)


    def refs_from(self, ea, flag):
        r"""
        Provide an iterator on from reference represented by flag
        """
        def _copy_xref():
            """ Make a private copy of the xref class to preserve its contents """
            class _xref(object):
                pass

            xr = _xref()
            for attr in [ 'frm', 'to', 'iscode', 'type', 'user' ]:
                setattr(xr, attr, getattr(self, attr))
            return xr

        if self.first_from(ea, flag):
            yield _copy_xref()
            while self.next_from():
                yield _copy_xref()


    def refs_to(self, ea, flag):
        r"""
        Provide an iterator on to reference represented by flag
        """
        def _copy_xref():
            """ Make a private copy of the xref class to preserve its contents """
            class _xref(object):
                pass

            xr = _xref()
            for attr in [ 'frm', 'to', 'iscode', 'type', 'user' ]:
                setattr(xr, attr, getattr(self, attr))
            return xr

        if self.first_to(ea, flag):
            yield _copy_xref()
            while self.next_to():
                yield _copy_xref()


    def __init__(self, *args):
        r"""
        __init__(self) -> xrefblk_t
        """
        _ida_xref.xrefblk_t_swiginit(self, _ida_xref.new_xrefblk_t(*args))
    __swig_destroy__ = _ida_xref.delete_xrefblk_t

# Register xrefblk_t in _ida_xref:
_ida_xref.xrefblk_t_swigregister(xrefblk_t)
XREF_ALL = _ida_xref.XREF_ALL
r"""
return all references
"""

XREF_FAR = _ida_xref.XREF_FAR
r"""
don't return ordinary flow xrefs
"""

XREF_DATA = _ida_xref.XREF_DATA
r"""
return data references only
"""



def get_first_dref_from(*args) -> "ea_t":
    r"""
    get_first_dref_from(frm) -> ea_t
    Get first data referenced from the specified address.

    @param from: (C++: ea_t) linear address of referencing instruction or data
    @return: linear address of first (lowest) data referenced from the specified
             address. Return BADADDR if the specified instruction/data doesn't
             reference to anything.
    """
    return _ida_xref.get_first_dref_from(*args)

def get_next_dref_from(*args) -> "ea_t":
    r"""
    get_next_dref_from(frm, current) -> ea_t
    Get next data referenced from the specified address.

    @param from: (C++: ea_t) linear address of referencing instruction or data
    @param current: (C++: ea_t) linear address of current referenced data. This value is
                    returned by get_first_dref_from() or previous call to
                    get_next_dref_from() functions.
    @return: linear address of next data or BADADDR.
    """
    return _ida_xref.get_next_dref_from(*args)

def get_first_dref_to(*args) -> "ea_t":
    r"""
    get_first_dref_to(to) -> ea_t
    Get address of instruction/data referencing to the specified data.

    @param to: (C++: ea_t) linear address of referencing instruction or data
    @return: BADADDR if nobody refers to the specified data.
    """
    return _ida_xref.get_first_dref_to(*args)

def get_next_dref_to(*args) -> "ea_t":
    r"""
    get_next_dref_to(to, current) -> ea_t
    Get address of instruction/data referencing to the specified data

    @param to: (C++: ea_t) linear address of referencing instruction or data
    @param current: (C++: ea_t) current linear address. This value is returned by
                    get_first_dref_to() or previous call to get_next_dref_to()
                    functions.
    @return: BADADDR if nobody refers to the specified data.
    """
    return _ida_xref.get_next_dref_to(*args)

def get_first_cref_from(*args) -> "ea_t":
    r"""
    get_first_cref_from(frm) -> ea_t
    Get first instruction referenced from the specified instruction. If the
    specified instruction passes execution to the next instruction then the next
    instruction is returned. Otherwise the lowest referenced address is returned
    (remember that xrefs are kept sorted!).

    @param from: (C++: ea_t) linear address of referencing instruction
    @return: first referenced address. If the specified instruction doesn't
             reference to other instructions then returns BADADDR.
    """
    return _ida_xref.get_first_cref_from(*args)

def get_next_cref_from(*args) -> "ea_t":
    r"""
    get_next_cref_from(frm, current) -> ea_t
    Get next instruction referenced from the specified instruction.

    @param from: (C++: ea_t) linear address of referencing instruction
    @param current: (C++: ea_t) linear address of current referenced instruction This value is
                    returned by get_first_cref_from() or previous call to
                    get_next_cref_from() functions.
    @return: next referenced address or BADADDR.
    """
    return _ida_xref.get_next_cref_from(*args)

def get_first_cref_to(*args) -> "ea_t":
    r"""
    get_first_cref_to(to) -> ea_t
    Get first instruction referencing to the specified instruction. If the specified
    instruction may be executed immediately after its previous instruction then the
    previous instruction is returned. Otherwise the lowest referencing address is
    returned. (remember that xrefs are kept sorted!).

    @param to: (C++: ea_t) linear address of referenced instruction
    @return: linear address of the first referencing instruction or BADADDR.
    """
    return _ida_xref.get_first_cref_to(*args)

def get_next_cref_to(*args) -> "ea_t":
    r"""
    get_next_cref_to(to, current) -> ea_t
    Get next instruction referencing to the specified instruction.

    @param to: (C++: ea_t) linear address of referenced instruction
    @param current: (C++: ea_t) linear address of current referenced instruction This value is
                    returned by get_first_cref_to() or previous call to
                    get_next_cref_to() functions.
    @return: linear address of the next referencing instruction or BADADDR.
    """
    return _ida_xref.get_next_cref_to(*args)

def get_first_fcref_from(*args) -> "ea_t":
    r"""
    get_first_fcref_from(frm) -> ea_t

    @param from: ea_t
    """
    return _ida_xref.get_first_fcref_from(*args)

def get_next_fcref_from(*args) -> "ea_t":
    r"""
    get_next_fcref_from(frm, current) -> ea_t

    @param from: ea_t
    @param current: ea_t
    """
    return _ida_xref.get_next_fcref_from(*args)

def get_first_fcref_to(*args) -> "ea_t":
    r"""
    get_first_fcref_to(to) -> ea_t

    @param to: ea_t
    """
    return _ida_xref.get_first_fcref_to(*args)

def get_next_fcref_to(*args) -> "ea_t":
    r"""
    get_next_fcref_to(to, current) -> ea_t

    @param to: ea_t
    @param current: ea_t
    """
    return _ida_xref.get_next_fcref_to(*args)

def has_external_refs(*args) -> "bool":
    r"""
    has_external_refs(pfn, ea) -> bool
    Does 'ea' have references from outside of 'pfn'?

    @param pfn: (C++: func_t *)
    @param ea: (C++: ea_t)
    """
    return _ida_xref.has_external_refs(*args)

def delete_switch_table(*args) -> "void":
    r"""
    delete_switch_table(jump_ea, si)

    @param jump_ea: ea_t
    @param si: switch_info_t const &
    """
    return _ida_xref.delete_switch_table(*args)
class casevec_t(object):
    r"""
    Proxy of C++ qvector< qvector< sval_t > > class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(self) -> casevec_t
        __init__(self, x) -> casevec_t

        @param x: qvector< qvector< long long > > const &
        """
        _ida_xref.casevec_t_swiginit(self, _ida_xref.new_casevec_t(*args))
    __swig_destroy__ = _ida_xref.delete_casevec_t

    def push_back(self, *args) -> "qvector< long long > &":
        r"""
        push_back(self, x)

        @param x: qvector< long long > const &

        push_back(self) -> qvector< long long > &
        """
        return _ida_xref.casevec_t_push_back(self, *args)

    def pop_back(self, *args) -> "void":
        r"""
        pop_back(self)
        """
        return _ida_xref.casevec_t_pop_back(self, *args)

    def size(self, *args) -> "size_t":
        r"""
        size(self) -> size_t
        """
        return _ida_xref.casevec_t_size(self, *args)

    def empty(self, *args) -> "bool":
        r"""
        empty(self) -> bool
        """
        return _ida_xref.casevec_t_empty(self, *args)

    def at(self, *args) -> "qvector< long long > const &":
        r"""
        at(self, _idx) -> qvector< long long > const &

        @param _idx: size_t
        """
        return _ida_xref.casevec_t_at(self, *args)

    def qclear(self, *args) -> "void":
        r"""
        qclear(self)
        """
        return _ida_xref.casevec_t_qclear(self, *args)

    def clear(self, *args) -> "void":
        r"""
        clear(self)
        """
        return _ida_xref.casevec_t_clear(self, *args)

    def resize(self, *args) -> "void":
        r"""
        resize(self, _newsize, x)

        @param _newsize: size_t
        @param x: qvector< long long > const &

        resize(self, _newsize)

        @param _newsize: size_t
        """
        return _ida_xref.casevec_t_resize(self, *args)

    def grow(self, *args) -> "void":
        r"""
        grow(self, x=qvector< long long >())

        @param x: qvector< long long > const &
        """
        return _ida_xref.casevec_t_grow(self, *args)

    def capacity(self, *args) -> "size_t":
        r"""
        capacity(self) -> size_t
        """
        return _ida_xref.casevec_t_capacity(self, *args)

    def reserve(self, *args) -> "void":
        r"""
        reserve(self, cnt)

        @param cnt: size_t
        """
        return _ida_xref.casevec_t_reserve(self, *args)

    def truncate(self, *args) -> "void":
        r"""
        truncate(self)
        """
        return _ida_xref.casevec_t_truncate(self, *args)

    def swap(self, *args) -> "void":
        r"""
        swap(self, r)

        @param r: qvector< qvector< long long > > &
        """
        return _ida_xref.casevec_t_swap(self, *args)

    def extract(self, *args) -> "qvector< long long > *":
        r"""
        extract(self) -> qvector< long long > *
        """
        return _ida_xref.casevec_t_extract(self, *args)

    def inject(self, *args) -> "void":
        r"""
        inject(self, s, len)

        @param s: qvector< long long > *
        @param len: size_t
        """
        return _ida_xref.casevec_t_inject(self, *args)

    def __eq__(self, *args) -> "bool":
        r"""
        __eq__(self, r) -> bool

        @param r: qvector< qvector< long long > > const &
        """
        return _ida_xref.casevec_t___eq__(self, *args)

    def __ne__(self, *args) -> "bool":
        r"""
        __ne__(self, r) -> bool

        @param r: qvector< qvector< long long > > const &
        """
        return _ida_xref.casevec_t___ne__(self, *args)

    def begin(self, *args) -> "qvector< qvector< long long > >::const_iterator":
        r"""
        begin(self) -> qvector< qvector< long long > >::iterator
        begin(self) -> qvector< qvector< long long > >::const_iterator
        """
        return _ida_xref.casevec_t_begin(self, *args)

    def end(self, *args) -> "qvector< qvector< long long > >::const_iterator":
        r"""
        end(self) -> qvector< qvector< long long > >::iterator
        end(self) -> qvector< qvector< long long > >::const_iterator
        """
        return _ida_xref.casevec_t_end(self, *args)

    def insert(self, *args) -> "qvector< qvector< long long > >::iterator":
        r"""
        insert(self, it, x) -> qvector< qvector< long long > >::iterator

        @param it: qvector< qvector< long long > >::iterator
        @param x: qvector< long long > const &
        """
        return _ida_xref.casevec_t_insert(self, *args)

    def erase(self, *args) -> "qvector< qvector< long long > >::iterator":
        r"""
        erase(self, it) -> qvector< qvector< long long > >::iterator

        @param it: qvector< qvector< long long > >::iterator

        erase(self, first, last) -> qvector< qvector< long long > >::iterator

        @param first: qvector< qvector< long long > >::iterator
        @param last: qvector< qvector< long long > >::iterator
        """
        return _ida_xref.casevec_t_erase(self, *args)

    def find(self, *args) -> "qvector< qvector< long long > >::const_iterator":
        r"""
        find(self, x) -> qvector< qvector< long long > >::iterator

        @param x: qvector< long long > const &

        find(self, x) -> qvector< qvector< long long > >::const_iterator

        @param x: qvector< long long > const &
        """
        return _ida_xref.casevec_t_find(self, *args)

    def has(self, *args) -> "bool":
        r"""
        has(self, x) -> bool

        @param x: qvector< long long > const &
        """
        return _ida_xref.casevec_t_has(self, *args)

    def add_unique(self, *args) -> "bool":
        r"""
        add_unique(self, x) -> bool

        @param x: qvector< long long > const &
        """
        return _ida_xref.casevec_t_add_unique(self, *args)

    def _del(self, *args) -> "bool":
        r"""
        _del(self, x) -> bool

        Parameters
        ----------
        x: qvector< long long > const &

        """
        return _ida_xref.casevec_t__del(self, *args)

    def __len__(self, *args) -> "size_t":
        r"""
        __len__(self) -> size_t
        """
        return _ida_xref.casevec_t___len__(self, *args)

    def __getitem__(self, *args) -> "qvector< long long > const &":
        r"""
        __getitem__(self, i) -> qvector< long long > const &

        @param i: size_t
        """
        return _ida_xref.casevec_t___getitem__(self, *args)

    def __setitem__(self, *args) -> "void":
        r"""
        __setitem__(self, i, v)

        @param i: size_t
        @param v: qvector< long long > const &
        """
        return _ida_xref.casevec_t___setitem__(self, *args)

    front = ida_idaapi._qvector_front
    back = ida_idaapi._qvector_back
    __iter__ = ida_idaapi._bounded_getitem_iterator


# Register casevec_t in _ida_xref:
_ida_xref.casevec_t_swigregister(casevec_t)


#<pycode(py_xref)>

import ida_idaapi
ida_idaapi._listify_types(
        casevec_t)

#</pycode(py_xref)>







"""
summary: using `ida_kernwin.UI_Hooks.get_chooser_item_attrs` to override some defaults

description:
  color the function in the Function window according to its size.
  The larger the function, the darker the color.
"""

import ida_kernwin
import ida_funcs
import math

class func_chooser_coloring_hooks_t(ida_kernwin.UI_Hooks):
    def __init__(self):
        ida_kernwin.UI_Hooks.__init__(self)
        self.colors = [0x808080 + (32-i) * 0x400 for i in range(32-5)]

    def get_chooser_item_attrs(self, chobj, n, attrs):
        if attrs.color != 0xFFFFFFFF:
            return # the color is already set
        ea = chobj.get_ea(n)
        fn = ida_funcs.get_func(ea)
        size = fn.size()
        if size < 32:
            return # do not color small functions
        attrs.color = self.colors[int(math.log2(size))]

fcch = func_chooser_coloring_hooks_t()
fcch.hook()
ida_kernwin.enable_chooser_item_attrs("Functions", True)

